#**************************************************************
# Time Information
#**************************************************************

set_time_format -unit ns -decimal_places 3

#**************************************************************
# Create Clock
#**************************************************************

create_clock -name {CLK40M} -period 25.000 -waveform { 0.000 12.500 } [get_ports {CLK40M}]

#**************************************************************
# Create Generated Clock
#**************************************************************

create_generated_clock -name {C_clk132_pll|t1_pll_132_inst|altera_pll_i|general[0].gpll~FRACTIONAL_PLL|vcoph[0]} -source [get_pins {C_clk132_pll|t1_pll_132_inst|altera_pll_i|general[0].gpll~FRACTIONAL_PLL|refclkin}] -duty_cycle 50/1 -multiply_by 66 -divide_by 5 -master_clock {CLK40M} [get_pins {C_clk132_pll|t1_pll_132_inst|altera_pll_i|general[0].gpll~FRACTIONAL_PLL|vcoph[0]}] 

create_generated_clock -name CLK_TOP -source [get_pins {C_clk132_pll|t1_pll_132_inst|altera_pll_i|general[1].gpll~PLL_OUTPUT_COUNTER|divclk}] -duty_cycle 50/1 -multiply_by 1 -divide_by 8 [get_pins {C_clk132_pll|t1_pll_132_inst|altera_pll_i|outclk_wire[1]~CLKENA0|outclk}]

create_generated_clock -name CLK_USB -source [get_pins {C_clk132_pll|t1_pll_132_inst|altera_pll_i|outclk_wire[1]~CLKENA0|outclk}] -duty_cycle 50/1 -multiply_by 1 -divide_by 1 [get_ports {CLK_USB}]

create_generated_clock -name ADC_CLK -source [get_pins {C_clk132_pll|t1_pll_132_inst|altera_pll_i|outclk_wire[1]~CLKENA0|outclk}] -divide_by 2 [get_pins {app|o_drs_adc_clk|clk}]

create_generated_clock -name ADC_FCO -source [get_pins {app|o_drs_adc_clk|clk}] [get_ports {ADC_FCO}]

create_generated_clock -name ADC_DCO -source [get_ports {ADC_FCO}] -multiply_by 6 [get_ports {ADC_DCO}]

# A test
# Set CLK_40M and CLK_USB to be mutually exclusive clocks.
# set_clock_groups -logically_exclusive -group {CLK40M} -group {CLK_USB} 

set period10ns 10

#**************************************************************
# Set Clock Latency
#**************************************************************

# see AD9637 data sheet
# tfco
set_clock_latency -source -late 2.3 [get_clocks {ADC_FCO}]  
# tframe = tsample/24 tsample=1/(33 MHz)
set_clock_latency -source -late 1.25 [get_clocks {ADC_DCO}]

#**************************************************************
# Set Clock Uncertainty
#**************************************************************

set_clock_uncertainty -fall_from [get_clocks {CLK40M}] -rise_to [get_clocks {CLK40M}] -setup 0.100  
set_clock_uncertainty -fall_from [get_clocks {CLK40M}] -rise_to [get_clocks {CLK40M}] -hold 0.060  
set_clock_uncertainty -fall_from [get_clocks {CLK40M}] -fall_to [get_clocks {CLK40M}] -setup 0.100  
set_clock_uncertainty -fall_from [get_clocks {CLK40M}] -fall_to [get_clocks {CLK40M}] -hold 0.060  

#USB DECLARATIONS ----------------------------------------
#pcb delay wrt PCLK, see J:\vhdl_db\trunk\projets\AID_AIDA_FEB\doc\PCB uc propagation delay.xlsx
set pcb_rel_delay_min -0.25
set pcb_rel_delay_max +0.15

#micro delay wrt PCLK : see CYUSB301X datasheet
set uc_clkToDinSetup 2
set uc_clkToDinHold 0.5

# Extra margin
set output_skew 0.125 

#Output Max Delay = uc read setup
set UcDinSetup [expr ($output_skew + $pcb_rel_delay_max + $uc_clkToDinSetup)]
post_message -type extra_info "USB uc Din Setup = $UcDinSetup ns."

#Output min Delay = uc read hold
set UcDinHold [expr ($uc_clkToDinHold + $pcb_rel_delay_min - $output_skew)]
post_message -type extra_info "USB uc Din Hold = $UcDinHold ns."

#CONSTRAINTS ----------------------------------------
set_output_delay $UcDinSetup -max -clock [get_clocks {CLK_USB}] [get_ports {DQ*}]
set_output_delay $UcDinHold -min -clock [get_clocks {CLK_USB}] [get_ports {DQ*}] -add_delay

#oe released constraint
set_max_delay -from [get_registers {usb3_racc:C_usb3_racc|uc_fdts}] -to [get_ports {DQ*}] $period10ns

set_output_delay $UcDinSetup -max -clock [get_clocks {CLK_USB}] [get_ports {SLWRn*}]
set_output_delay $UcDinHold -min -clock [get_clocks {CLK_USB}] [get_ports {SLWRn*}] -add_delay

set_output_delay $UcDinSetup -max -clock [get_clocks {CLK_USB}] [get_ports {SLRDn*}]
set_output_delay $UcDinHold -min -clock [get_clocks {CLK_USB}] [get_ports {SLRDn*}] -add_delay

set_output_delay $UcDinSetup -max -clock [get_clocks {CLK_USB}] [get_ports {SLOEn*}]
set_output_delay $UcDinHold -min -clock [get_clocks {CLK_USB}] [get_ports {SLOEn*}] -add_delay

set_output_delay $UcDinSetup -max -clock [get_clocks {CLK_USB}] [get_ports {SLCSn*}]
set_output_delay $UcDinHold -min -clock [get_clocks {CLK_USB}] [get_ports {SLCSn*}] -add_delay

set_output_delay $UcDinSetup -max -clock [get_clocks {CLK_USB}] [get_ports {A[*}]
set_output_delay $UcDinHold -min -clock [get_clocks {CLK_USB}] [get_ports {A[*}] -add_delay

set_output_delay $UcDinSetup -max -clock [get_clocks {CLK_USB}] [get_ports {PKTENDn*}]
set_output_delay $UcDinHold -min -clock [get_clocks {CLK_USB}] [get_ports {PKTENDn*}] -add_delay

set_output_delay $UcDinSetup -max -clock [get_clocks {CLK_USB}] [get_ports {UC_RESETn*}]
set_output_delay $UcDinHold -min -clock [get_clocks {CLK_USB}] [get_ports {UC_RESETn*}] -add_delay

#**************************************************************
# Set Input Delay
#**************************************************************

#USB DECLARATIONS ----------------------------------------
#use for clk100=PCLK
set din_latch 0 
#use for clk100=not(PCLK)
#set din_latch [expr $period10ns] 

#PLK delay : see J:\vhdl_db\trunk\projets\AID_AIDA_FEB\doc\PCB uc propagation delay.xlsx
set pcb_pclk_delay 0.52

#micro delay wrt PCLK : see CYUSB301X datasheet
#set uc_clkToValidDout 8
set uc_clkToValidDout 7.5
set uc_clkToDoutHold 2

#Input Max Delay = fpga read setup 
set DinSetup [expr ($pcb_pclk_delay + $pcb_rel_delay_max + $uc_clkToValidDout + $output_skew - $din_latch)]
post_message -type extra_info "USB FPGA Din Setup = $DinSetup ns."

#Input min Delay = fpga read hold
set DinHold [expr ($pcb_pclk_delay + $pcb_rel_delay_min + $uc_clkToDoutHold - $output_skew - $din_latch)]
post_message -type extra_info "USB FPGA Din Hold = $DinHold ns."

#CONSTRAINTS ----------------------------------------
set_input_delay $DinSetup -max -clock [get_clocks {CLK_USB}] [get_ports {GPIO51}]
set_input_delay $DinHold -min -clock [get_clocks {CLK_USB}] [get_ports {GPIO51}] -add_delay

set_input_delay $DinSetup -max -clock [get_clocks {CLK_USB}] [get_ports {FLAG*}]
set_input_delay $DinHold -min -clock [get_clocks {CLK_USB}] [get_ports {FLAG*}] -add_delay
 
set_input_delay $DinSetup -max -clock [get_clocks {CLK_USB}] [get_ports {CTL6}]
set_input_delay $DinHold -min -clock [get_clocks {CLK_USB}] [get_ports {CTL6}] -add_delay
 
set_input_delay $DinSetup -max -clock [get_clocks {CLK_USB}] [get_ports {CTL8}]
set_input_delay $DinHold -min -clock [get_clocks {CLK_USB}] [get_ports {CTL8}] -add_delay
 
set_input_delay $DinSetup -max -clock [get_clocks {CLK_USB}] [get_ports {DQ*}]
set_input_delay $DinHold -min -clock [get_clocks {CLK_USB}] [get_ports {DQ*}] -add_delay








