--##################################################################
-- Author   : Boris Keil, Stefan Ritt
-- Contents : User FPGA library with various components
-- $Id: usr_lib.vhd 6908 2007-03-06 07:15:16Z ritt $
--##################################################################
-- 20160912 modified by Wolfgang.Rauch@cern.ch


-----------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.ALL;

entity ibufreg is
  generic (
    width : integer := 1
  );
  port (
    CLK  : in  std_logic;
    PRE  : in  std_logic;
    IPIN : in  std_logic_vector(width-1 downto 0);
    O    : out std_logic_vector(width-1 downto 0)
  );
end ibufreg;

architecture struct of ibufreg is

component ibufddr1
  port(
    aset      : in  std_logic;
    datain    : in  std_logic_vector(0 downto 0);
    inclock   : in  std_logic;
    dataout_h : out std_logic_vector(0 downto 0);
    dataout_l : out std_logic_vector(0 downto 0)
  );
end component;

begin

  gen : for count in 0 to width-1 generate
  begin
    U1 : ibufddr1
      port map (
        aset      => PRE,
        datain    => IPIN(count downto count),
        inclock   => CLK,
        dataout_h => O(count downto count),
        dataout_l => open
      );
  end generate;

end struct;

-----------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.ALL;

entity ibufreg1 is
  port (
    CLK  : in  std_logic;
    PRE  : in  std_logic;
    IPIN : in  std_logic;
    O    : out std_logic
  );
end ibufreg1;

architecture struct of ibufreg1 is

component ibufddr1
  port(
    aset      : in  std_logic;
    datain    : in  std_logic_vector(0 downto 0);
    inclock   : in  std_logic;
    dataout_h : out std_logic_vector(0 downto 0);
    dataout_l : out std_logic_vector(0 downto 0)
  );
end component;

signal datain    : std_logic_vector(0 downto 0);
signal dataout_h : std_logic_vector(0 downto 0);

begin

datain(0) <= IPIN;
O <= dataout_h(0);

  U1 : ibufddr1
    port map (
      aset      => PRE,
      datain    => datain,
      inclock   => CLK,
      dataout_h => dataout_h,
      dataout_l => open
    );

end struct;

-----------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.ALL;

entity ibufreg1r is
  port (
    CLK  : in  std_logic;
    CLR  : in  std_logic;
    IPIN : in  std_logic;
    O    : out std_logic
  );
end ibufreg1r;

architecture struct of ibufreg1r is

component ibufddr1r
  port(
    aclr      : in  std_logic;
    datain    : in  std_logic_vector(0 downto 0);
    inclock   : in  std_logic;
    dataout_h : out std_logic_vector(0 downto 0);
    dataout_l : out std_logic_vector(0 downto 0)
  );
end component;

signal datain    : std_logic_vector(0 downto 0);
signal dataout_h : std_logic_vector(0 downto 0);

begin

datain(0) <= IPIN;
O <= dataout_h(0);

  U1 : ibufddr1r
    port map (
      aclr      => CLR,
      datain    => datain,
      inclock   => CLK,
      dataout_h => dataout_h,
      dataout_l => open
    );

end struct;

-----------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.ALL;

entity obufreg is
  generic (
    width : integer := 1
  );
  port (
    CLK  : in  std_logic;
    PRE  : in  std_logic;
    I    : in  std_logic_vector(width-1 downto 0);
    OPIN : out std_logic_vector(width-1 downto 0)
  );
end obufreg;

architecture struct of obufreg is

component obufddr1
  port(
    aset     : in  std_logic;
    datain_h : in  std_logic_vector(0 downto 0);
    datain_l : in  std_logic_vector(0 downto 0);
    outclock : in  std_logic;
    dataout  : out std_logic_vector(0 downto 0)
  );
end component;

begin

  gen : for count in 0 to width-1 generate
  begin
    U1 : obufddr1
      port map (
        aset     => PRE,
        datain_h => I(count downto count),
        datain_l => I(count downto count),
        outclock => CLK,
        dataout  => OPIN(count downto count)
      );
  end generate;

end struct;
-----------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.ALL;

entity obufreg1 is
  port (
    CLK  : in  std_logic;
    PRE  : in  std_logic;
    I    : in  std_logic;
    OPIN : out std_logic
  );
end obufreg1;

architecture struct of obufreg1 is

component obufddr1
  port(
    aset     : in  std_logic;
    datain_h : in  std_logic_vector(0 downto 0);
    datain_l : in  std_logic_vector(0 downto 0);
    outclock : in  std_logic;
    dataout  : out std_logic_vector(0 downto 0)
  );
end component;

signal datain  : std_logic_vector(0 downto 0);
signal dataout : std_logic_vector(0 downto 0);

begin

datain(0) <= I;
OPIN <= dataout(0);

  U1 : obufddr1
    port map (
      aset     => PRE,
      datain_h => datain,
      datain_l => datain,
      outclock => CLK,
      dataout  => dataout
    );

end struct;

-----------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.ALL;
use ieee.std_logic_unsigned.all;

entity deserf12 is
  port (
    CLK    : in  std_logic;         -- FIFO clock, same as ADC clock (sample clock)
    DCO    : in  std_logic;         -- Data clock from ADC, uses both edges
    FCO    : in  std_logic;         -- Frame clock from ADC
    PRE    : in  std_logic;
    D      : in  std_logic;         -- Serial data from ADC
    Q      : out std_logic_vector(11 downto 0)
  );
end deserf12;

architecture struct of deserf12 is

component ibufddr1
  port(
    aset      : in  std_logic;
    datain    : in  std_logic_vector(0 downto 0);
    inclock   : in  std_logic;
    dataout_h : out std_logic_vector(0 downto 0);
    dataout_l : out std_logic_vector(0 downto 0)
  );
end component;

constant n_bits_drs_adc : positive := 12;

signal data_h : std_logic_vector(0 downto 0);
signal data_l : std_logic_vector(0 downto 0);
signal datain : std_logic_vector(0 downto 0);
signal data   : std_logic_vector((n_bits_drs_adc*2-1) downto 0); -- we memorize two ADC frames

begin

datain(0) <= D;

  U1 : ibufddr1
    port map (
      aset      => PRE,
      datain    => datain,
      inclock   => DCO,
      dataout_h => data_h,
      dataout_l => data_l
    );

-- fast clock
process(DCO)
begin
  if rising_edge(DCO) then
-- data_h and data_l may have to be swapped
    data(data'high downto 0) <= data((data'high-2) downto 0) & data_l(0) & data_h(0);
  end if;
end process;

-- slow clock (frame clock)
process(PRE, FCO)
begin
  if (PRE='1') then
    Q     <= X"666";
  elsif rising_edge(FCO) then
    Q     <= data((data'high-2) downto n_bits_drs_adc-2); -- previous frame ok, so we are one frame behind
  end if;
end process;

end struct;

-----------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.ALL;
use work.dpnc342_pack.all;

entity deserf12_a is
  generic (
    width : integer := 1
  );
  port (
    CLK    : in  std_logic;         -- FIFO clock, same as ADC clock (sample clock)
    DCO    : in  std_logic;         -- Data clock from ADC, uses both edges
    FCO    : in  std_logic;         -- Frame clock from ADC
    PRE    : in  std_logic;
    D      : in  std_logic_vector((width-1) downto 0);  -- Serial data from ADC
    Q      : out t_drs_adc_val_a((width-1) downto 0)
  );
end deserf12_a;

architecture struct of deserf12_a is

component deserf12 is
  port (
    CLK    : in  std_logic;         -- FIFO clock, same as ADC clock (sample clock)
    DCO    : in  std_logic;         -- Data clock from ADC, uses both edges
    FCO    : in  std_logic;         -- Frame clock from ADC
    PRE    : in  std_logic;
    D      : in  std_logic;         -- Serial data from ADC
    Q      : out std_logic_vector(11 downto 0)
  );
end component deserf12;

begin
  gen : for count in 0 to width-1 generate
  begin
    U1 : deserf12
      port map (
        CLK    => CLK,
        DCO    => DCO,
        FCO    => FCO,
        PRE    => PRE,
        D      => D(count),
        Q      => Q(count)
      );
  end generate;

end struct;

