-- Inherited from S. Ritt et al.
-- Modified by S. Debieux
-- 20160912 further modified by Wolfgang.Rauch@cern.ch
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use IEEE.NUMERIC_STD.ALL;
use work.dpnc342_pack.all;

entity T1_top is
  port(-- MCU signals
       FLAGA        : in    std_logic;
       FLAGB        : in    std_logic;
       DQ           : inout std_logic_vector(31 downto 0);
       A            : out   std_logic_vector(1 downto 0);
       PKTENDn      : out   std_logic;
       SLWRn        : out   std_logic;
       SLRDn        : out   std_logic;
       SLOEn        : out   std_logic;
       SLCSn        : out   std_logic;
       CLK_USB      : out   std_logic;
       UC_UART_RTSn : in    std_logic;   -- not used yet
       UC_UART_CTSn : out   std_logic;   -- not used yet
       UC_UART_TX   : in    std_logic;   -- not used yet
       UC_UART_RX   : out   std_logic;   -- not used yet
       CTL6         : inout std_logic;
       CTL8         : inout std_logic;
       CTL9         : inout std_logic;   -- set to High impedance
       CTL10        : inout std_logic;   -- set to High impedance
       GPIO50       : inout std_logic;   -- set to High impedance
       GPIO51       : inout std_logic;
       UC_RESETn    : out   std_logic;
       INTn         : out   std_logic;   -- set to High impedance
       -- DRS4 signals
       DRS_A        : out   std_logic_vector(3 downto 0);
       DRS_RSRLOAD  : out   std_logic;
       DRS_SRCLK    : out   std_logic;
       DRS_SRIN     : out   std_logic;
       DRS_DWRITE   : out   std_logic;
       DRS_DENABLE  : out   std_logic;
       DRS_WSRIN    : out   std_logic;      -- not used, no cascading/daisy-chaining

       DRS1_SROUT   : in    std_logic;
       DRS1_WSROUT  : in    std_logic;
       DRS1_PLLLCK  : in    std_logic;

       DRS2_SROUT   : in    std_logic;
       DRS2_WSROUT  : in    std_logic;
       DRS2_PLLLCK  : in    std_logic;

       DRS3_SROUT   : in    std_logic;
       DRS3_WSROUT  : in    std_logic;
       DRS3_PLLLCK  : in    std_logic;

       DRS4_SROUT   : in    std_logic;
       DRS4_WSROUT  : in    std_logic;
       DRS4_PLLLCK  : in    std_logic;
       -- Temperature Sensors (DRS4)
       TS_SDO       : in    std_logic;
       TS_SDI       : out   std_logic;
       TS_SCK       : out   std_logic;
       TS1CS        : out   std_logic;
       TS2CS        : out   std_logic;
       TS3CS        : out   std_logic;
       TS4CS        : out   std_logic;
       -- ADC signals
       ADC_CLK      : out   std_logic;      -- WARNING: differential output (LVDS)
       ADC_DCO      : in    std_logic;      -- WARNING: differential input (LVDS)
       ADC_FCO      : in    std_logic;      -- WARNING: differential input (LVDS)
       ADC_OUT1     : in    std_logic;      -- WARNING: differential input (LVDS)
       ADC_OUT2     : in    std_logic;      -- WARNING: differential input (LVDS)
       ADC_OUT3     : in    std_logic;      -- WARNING: differential input (LVDS)
       ADC_OUT4     : in    std_logic;      -- WARNING: differential input (LVDS)
       ADC_CSB      : out   std_logic;
       ADC_SCLK     : out   std_logic;
       ADC_SDIO     : inout std_logic;
       -- Analog Front-End signals
       TRIG         : in    std_logic_vector(4 downto 1);
       SEL_CAL      : out   std_logic;
       SEL_IN       : out   std_logic;
       SEL_TC       : out   std_logic;
       CXOE         : out   std_logic;
       VCALDIS      : out   std_logic;
       TCAE         : out   std_logic;
       -- DAC signals
       DAC_CS       : out   std_logic;
       DAC_SCK      : out   std_logic;
       DAC_SDI      : out   std_logic;
       -- Miscellaneous signals
       TRIGIN       : in    std_logic_vector(1 to 4);
       TRIGIN_FP    : in    std_logic;
       BUSY_FP      : out   std_logic;
       STAT         : out   std_logic_vector(1 to 4);
       RESETOUT     : out   std_logic;                     -- DRS4 Reset
       RESETIN      : in    std_logic;                     -- Active-High Hardware Reset (Power-On)
       LED          : out   std_logic_vector(7 downto 0);
       CLK40M       : in    std_logic;
       EECS         : out   std_logic;                     -- Board EEPROM Chip Select (SPI)
       BOARDID      : in    std_logic_vector(11 downto 0);
       -- DDL Link
       DDL_DATIN    : in    std_logic;
       DDL_CLKIN    : in    std_logic;
       DDL_DATOUT   : out   std_logic;
       DDL_CLKOUT   : out   std_logic;
       DDL_EN       : out   std_logic;
       -- User I/O LVDS
       UTXEN        : out   std_logic;
       UTX          : out   std_logic_vector(7 downto 0);
       URXEN        : out   std_logic;
       URX          : in    std_logic_vector(7 downto 0);
       -- Test points
       TESTIO       : out   std_logic_vector(15 downto 0)
      );
end T1_top;

--------------------------------------------------------------------------------
architecture struct of T1_top is

---------------------- constants ----------------------------------
 
  component T1_pll_132 is
    port (
      refclk   : in  std_logic := '0'; --  refclk.clk     40 MHz
      rst      : in  std_logic := '0'; --  reset.reset
      outclk_0 : out std_logic;        --  outclk0.clk   132 MHz
      outclk_1 : out std_logic;        --  outclk1.clk    66 MHz
      outclk_2 : out std_logic;        --  outclk2.clk    33 MHz
      locked   : out std_logic         --  locked.export
    );
  end component T1_pll_132;

  component T1_pll_1 is
    port (
      refclk   : in  std_logic := 'X'; -- clk
      rst      : in  std_logic := 'X'; -- reset
      outclk_0 : out std_logic;        -- clk
      locked   : out std_logic         -- export
    );
  end component T1_pll_1;

  component ledpulse
    port(
      PULSE  : in  std_logic;
      LEDOUT : out std_logic;
      CLK    : in  std_logic;
      RESET  : in  std_logic
    );
  end component;

  component spi_adc
    port(
      CSB   : out   std_logic;
      SCLK  : out   std_logic;
      SDIO  : inout std_logic;
      SRST  : in    std_logic;
      CLK   : in    std_logic;
      RESET : in    std_logic
    );
  end component;


  component usb3_racc
  generic(
    x_uc_data_bits        : positive := 32;       -- uc interface datasize (must be 32 or 16)
    x_uc_dma_in_nb_w32    : positive := n_w32_usb_dma_in;  -- uc DMA EP IN  buffer size (32-bit words)            
    x_uc_dma_out_nb_w32   : positive := n_w32_usb_dma_out; -- uc DMA EP OUT buffer size (32-bit words)            
    x_reset_active        : std_logic := '0'
    );
  port (
    x_clkin    : in std_logic;
    x_nreset   : in std_logic;
-- uC ports
    x_ucDQ     : inout std_logic_vector((x_uc_data_bits-1) downto 0);     -- uC FIFO slave data
    x_ucPCLK   : out   std_logic;       -- FX3 FIFO slave clock
    x_ucSLCSn  : out   std_logic;       -- FX3 FIFO slave chip select
    x_ucSLWRn  : out   std_logic;       -- FX3 FIFO slave write strobe
    x_ucSLRDn  : out   std_logic;       -- FX3 FIFO slave read strobe
    x_ucSLOEn  : out   std_logic;       -- FX3 FIFO slave output enable
    x_ucPKENDn : out   std_logic;       -- FX3 FIFO slave packet end
    x_ucADR    : out   std_logic_vector(1 downto 0);    -- FX3 FIFO slave address bus
    x_ucFLAG   : in    std_logic_vector(3 downto 0);    -- FX3 FIFO slave flags
    x_ucSTARTn : in    std_logic;       -- uC IO start signal   

-- Simple bus interface to on-chip RAM
-- --------------------------------------------------
    O_LOCBUS_ADDR           : out std_logic_vector(31 downto 0);
    I_LOCBUS_D_RD           : in  std_logic_vector(31 downto 0);
    O_LOCBUS_D_WR           : out std_logic_vector(31 downto 0);
    O_LOCBUS_WE             : out std_logic;
--
-- Status & control registers
-- --------------------------
    O_CONTROL_REG_ARR       : out t_reg_ctrl_a;
    I_STATUS_REG_ARR        : in  t_reg_stat_a;

    O_BYTESEL_ARR           : out t_bytesel_a;
    O_BITSET_VEC            : out t_bitset_v;

-- information
    x_ready    : out std_logic
--debug
--    x_debug    : out std_logic_vector((DEBUG_SIZE-1) downto 0)
    );
  end component;

  component dpram_4k32 is
  port (
    address_a       : in std_logic_vector ((n_addr_bits_w32_dpram-1) downto 0);
    address_b       : in std_logic_vector ((n_addr_bits_w32_dpram-1) downto 0);
    clock_a         : in std_logic :='1';
    clock_b         : in std_logic :='1';
    data_a          : in std_logic_vector (31 downto 0);
    data_b          : in std_logic_vector (31 downto 0);
    wren_a          : in std_logic  := '0';
    wren_b          : in std_logic  := '0';
    q_a             : out std_logic_vector (31 downto 0);
    q_b             : out std_logic_vector (31 downto 0)
    );
  end component dpram_4k32;

  component dpram_16k32 is
  port (
    address_a       : in std_logic_vector ((n_addr_bits_w32_dpram_eeprom-1) downto 0);
    address_b       : in std_logic_vector ((n_addr_bits_w32_dpram_eeprom-1) downto 0);
    clock_a         : in std_logic :='1';
    clock_b         : in std_logic :='1';
    data_a          : in std_logic_vector (31 downto 0);
    data_b          : in std_logic_vector (31 downto 0);
    wren_a          : in std_logic  := '0';
    wren_b          : in std_logic  := '0';
    q_a             : out std_logic_vector (31 downto 0);
    q_b             : out std_logic_vector (31 downto 0)
    );
  end component dpram_16k32;

  
  component drs4_eval5_app
  port (
-- clocks
    I_DRS_READOUT_CLK       : in std_logic;
    I_RESET                 : in std_logic; -- active high power-up reset

-- calibration
    O_CAL                   : out std_logic;
    O_TCA_CTRL              : out std_logic;
    O_TCA_SEL               : out std_logic;

-- analog triggers
    I_ANA_TRG               : in std_logic_vector(3 downto 0);

-- external trigger
    I_ETRG_IN               : in  std_logic;

    O_ETRG_OUT              : out std_logic;

    I_ADC_DCO               : in  std_logic;
    I_ADC_FCO               : in  std_logic;
    I_ADC_OUT               : in  std_logic_vector(3 downto 0);
    O_ADC_CLK               : out std_logic;

    O_DRS_DENABLE           : out std_logic;
    O_DRS_DWRITEx           : out std_logic;
    O_DRS_SRINx             : out std_logic;
    O_DRS_SRCLKx            : out std_logic;
    O_DRS_RSRLOADx          : out std_logic;
    O_DRS_A                 : out std_logic_vector(3 downto 0);

    I_DRS_WSROUTx           : in  std_logic_vector(3 downto 0);
    I_DRS_SROUTx            : in  std_logic_vector(3 downto 0);
    I_DRS_PLLLCKx           : in  std_logic_vector(3 downto 0);

    I_TS_SDO                : in  std_logic;
    O_TS_SDI                : out std_logic;
    O_TS_SCK                : out std_logic;
    O_TS_CS                 : out std_logic_vector(3 downto 0);

    O_DAC_CS                : out std_logic;
    O_DAC_SCK               : out std_logic;
    O_DAC_SDI               : out std_logic;

    O_EEPROM_CS             : out std_logic;

    -- Simple bus interface to DPRAM
    -- Wolfgang.rauch@cern.ch
    -- 64k eeprom dpram
    O_DPRAM_CLK        : out std_logic;

    O_DPRAM_EEPROM_ADDR       : out std_logic_vector(n_addr_bits_w32_dpram_eeprom-1+2 downto 0);
    O_DPRAM_EEPROM_D_WR       : out std_logic_vector(31 downto 0);
    O_DPRAM_EEPROM_WE         : out std_logic;
    I_DPRAM_EEPROM_D_RD       : in std_logic_vector(31 downto 0);


    -- Wolfgang.rauch@cern.ch
    -- 4 x 4x4k@32bit  wave data dpram (8k@16bit)
    -- 4 x 4x4k@32bit  pedestal dpram (8k@16bit)
    O_DPRAM_ADDR       : out t_dpram_readout_addr_a(7 downto 0);
    O_DPRAM_D_WR       : out t_dpram_readout_d_wr_a(7 downto 0);
    O_DPRAM_WE         : out std_logic_vector(7 downto 0);
    I_DPRAM_D_RD       : in t_dpram_readout_d_rd_a(7 downto 0);

      -- Control & status registers from system FPGA interface
    I_CONTROL_REG_ARR       : in  t_reg_ctrl_a;
    O_STATUS_REG_ARR        : out t_reg_stat_a;
    I_BYTESEL_ARR           : in  t_bytesel_a;
    I_BITSET_VEC            : in  t_bitset_v

    );
  end component;

  constant C_FW_VER : std_logic_vector(15 downto 0):=X"0001";
  constant C_HW_VER : std_logic_vector(15 downto 0):=X"0000";

  -- array of saved dpram addresses
  constant n_locbus_addr_saved : positive := 2;
  type t_locbus_addr_a is array (natural range <>) of std_logic_vector (31 downto 0);

  -- reset signal
  -- -------------
  signal global_reset : std_logic;  -- active high power-up reset
  
  -- clocks & related signals
  -- ------------------------
  signal clk_pllout : std_logic;
--  signal clk_cnt    : std_logic_vector(1 downto 0);
  signal clk_top    : std_logic;
  signal clk_adc    : std_logic;
  signal pll_locked : std_logic;

  -- user application signals for Locbus interface
  -- ---------------------------------------------
  signal locbus_addr         : std_logic_vector(31 downto 0);
  signal locbus_addr_saved   : t_locbus_addr_a((n_locbus_addr_saved-1) downto 0);
  signal locbus_d_rd : std_logic_vector(31 downto 0);
  signal locbus_d_wr : std_logic_vector(31 downto 0);
  signal locbus_we   : std_logic;
  signal locbus_eeprom_we   : std_logic;
  signal locbus_dpram_we    : std_logic_vector(7 downto 0);
  signal q_a_locbus_eeprom      : std_logic_vector(31 downto 0);
  signal q_a_locbus_readout     : t_dpram_readout_d_rd_a(7 downto 0);

  -- user application signals for DPRAM interface
  -- --------------------------------------------
  signal dpram_clk              : std_logic;
  signal dpram_i_reset          : std_logic;
  signal dpram_we               : std_logic_vector(7 downto 0);
  signal dpram_addr             : t_dpram_readout_addr_a(7 downto 0);
  signal dpram_d_wr             : t_dpram_readout_d_wr_a(7 downto 0);
  signal dpram_d_rd             : t_dpram_readout_d_rd_a(7 downto 0);
  signal dpram_eeprom_reset     : std_logic;
  signal dpram_eeprom_we        : std_logic;
  signal dpram_eeprom_addr      : std_logic_vector(n_addr_bits_w32_dpram_eeprom-1+2 downto 0);
  signal dpram_eeprom_d_wr      : std_logic_vector(31 downto 0);
  signal dpram_eeprom_d_rd      : std_logic_vector(31 downto 0);

  -- register signals for data exchange with microcontroller
  -- -------------------------------------------------------
  signal control_reg_arr       : t_reg_ctrl_a;
  signal status_reg_arr        : t_reg_stat_a;

  signal bytesel_arr           : t_bytesel_a;
  signal bitset_vec            : t_bitset_v;

  signal my_slwrn              : std_logic;
  signal my_slrdn              : std_logic;

  -- register signals for data exchange with DRS
  -- -------------------------------------------------------
  signal my_etrg        : std_logic;

--  signal o_debug : std_logic_vector(3 downto 0);
   
  signal tca_ctrl : std_logic;
  signal tcal_sel : std_logic;
  signal vcal     : std_logic;
  
  signal rw_reg : std_logic;
  signal rw_ram : std_logic;

  ----------------------------------------------

  signal logic0 : std_logic;

  signal ucflag       : std_logic_vector(3 downto 0);

  signal BOARDID_inv : std_logic_vector(11 downto 0);

  signal led_ready   : std_logic;
  signal led_acquire : std_logic;
  signal led_busy    : std_logic;
  signal led_error   : std_logic;
  signal led_trigin  : std_logic;
  signal led_datatx  : std_logic;
  signal led_tcal    : std_logic;
  signal led_fwupd   : std_logic;
--  signal fifo_fill   : std_logic;

  signal x_ready_d   : std_logic;
--  signal debug       : std_logic_vector((DEBUG_SIZE-1) downto 0);
begin
--------------------------------------------------------------------------------

-- Component Instantiation ----------------------------------- 

  C_clk132_pll : component T1_pll_132
    port map (
      refclk   => CLK40M,         --  refclk.clk     40 MHz
      rst      => RESETIN,        --  reset.reset
      outclk_0 => clk_pllout,     --  outclk0.clk   132 MHz        NOT USED
      outclk_1 => clk_top,        --  outclk1.clk    66 MHz
      outclk_2 => clk_adc,        --  outclk2.clk    33 MHz        NOT USED
      locked   => pll_locked      --  locked.export
    );

-- ADC SPI: during the initialization, the ADC needs a stable clock. If not, a digital reset through SPI is needed.
-- Reset command is automatically generated after Power-on reset.
  spiadc0: spi_adc
    port map (
      CSB   => ADC_CSB,
      SCLK  => ADC_SCLK,
      SDIO  => ADC_SDIO,
      SRST  => logic0,      -- not used
      CLK   => CLK40M,
      RESET => global_reset
    );

  led0 : ledpulse
    port map (
      PULSE  => led_ready,
      LEDOUT => LED(0),               -- READY (Green)
      CLK    => clk_top,
      RESET  => global_reset
    );

  led1 : ledpulse
    port map (
      PULSE  => led_acquire,
      LEDOUT => LED(1),               -- ACQUIRE (Green)
      CLK    => clk_top,
      RESET  => global_reset
    );

  led2 : ledpulse
    port map (
      PULSE  => led_busy,
      LEDOUT => LED(2),               -- BUSY (Green)
      CLK    => clk_top,
      RESET  => global_reset
    );

  led3 : ledpulse
    port map (
      PULSE  => led_error,
      LEDOUT => LED(3),               -- ERROR (Green)
      CLK    => clk_top,
      RESET  => global_reset
    );

  led4 : ledpulse
    port map (
      PULSE  => led_trigin,
      LEDOUT => LED(4),               -- TRIG IN (Green)
      CLK    => clk_top,
      RESET  => global_reset
    );

  led5 : ledpulse
    port map (
      PULSE  => led_datatx,
      LEDOUT => LED(5),               -- DATA TRANSFER (Green)
      CLK    => clk_top,
      RESET  => global_reset
    );

  led6 : ledpulse
    port map (
      PULSE  => led_tcal,
      LEDOUT => LED(6),               -- TIMING CAL (Green)
      CLK    => clk_top,
      RESET  => global_reset
    );

  led7 : ledpulse
    port map (
      PULSE  => led_fwupd,
      LEDOUT => LED(7),               -- FW UPDATE (Green)
      CLK    => clk_top,
      RESET  => global_reset
    );

--
-- DPRAM (FIFO) for waveform storage
-- ---------------------------------
  C_drs_dpram_eeprom : dpram_16k32
    port map (
      clock_a   => clk_top,
      address_a => locbus_addr((n_addr_bits_w32_dpram_eeprom-1+2) downto 2), -- 32-bit word addressing
      wren_a    => locbus_eeprom_we,
      q_a       => q_a_locbus_eeprom,
      data_a    => locbus_d_wr,

      clock_b   => dpram_clk,
      address_b => dpram_eeprom_addr((n_addr_bits_w32_dpram_eeprom-1+2) downto 2),
      wren_b    => dpram_eeprom_we,
      q_b       => dpram_eeprom_d_rd,
      data_b    => dpram_eeprom_d_wr
    );

  gen_dpram_readout: for i in 0 to 7 generate
  U0 : dpram_4k32
    port map (
      clock_a   => clk_top,
      address_a => locbus_addr((n_addr_bits_w32_dpram-1+2) downto 2), -- 32-bit word addressing
      wren_a    => locbus_dpram_we(i),
      q_a       => q_a_locbus_readout(i),
      data_a    => locbus_d_wr,

      clock_b   => dpram_clk,
      address_b => dpram_addr(i)((n_addr_bits_w32_dpram-1+2) downto 2),
      wren_b    => dpram_we(i),
      q_b       => dpram_d_rd(i),
      data_b    => dpram_d_wr(i)
    );
  end generate gen_dpram_readout;

  C_usb3_racc: usb3_racc
  generic map(
    x_uc_data_bits        => 32,                   -- uc interface datasize (must be 32 or 16)
    x_uc_dma_in_nb_w32    => n_w32_usb_dma_in,     -- uc DMA in nb            
    x_uc_dma_out_nb_w32   => n_w32_usb_dma_out,    -- uc DMA out nb            
    x_reset_active        => '0'
    )
  port map(
    x_clkin    => clk_top,
    x_nreset   => pll_locked,
-- uC ports
    x_ucDQ     => DQ,
    x_ucPCLK   => CLK_USB,
    x_ucSLCSn  => SLCSn,
    x_ucSLWRn  => my_slwrn,
    x_ucSLRDn  => my_slrdn,
    x_ucSLOEn  => SLOEn,
    x_ucPKENDn => PKTENDn,
    x_ucADR    => A,
    x_ucFLAG   => ucflag,
    x_ucSTARTn => GPIO51,

-- Simple bus interface to on-chip RAM
-- --------------------------------------------------
    O_LOCBUS_ADDR            => locbus_addr,
    I_LOCBUS_D_RD            => locbus_d_rd,
    O_LOCBUS_D_WR            => locbus_d_wr,
    O_LOCBUS_WE              => locbus_we,               
                                                           
-- Status & control registers
-- --------------------------
    O_CONTROL_REG_ARR        => control_reg_arr,
    I_STATUS_REG_ARR         => status_reg_arr,
                                                           
    O_BYTESEL_ARR            => bytesel_arr,
    O_BITSET_VEC             => bitset_vec,   
      
-- USB state machine information
    x_ready    => x_ready_d
    );

  app : drs4_eval5_app
    port map (
      I_RESET                 => global_reset,
      I_DRS_READOUT_CLK       => clk_top,
--      I_CLK66                 => clk_pllout,

      O_CAL                   => vcal,
      O_TCA_CTRL              => tca_ctrl,
      O_TCA_SEL               => tcal_sel,

      I_ETRG_IN               => TRIGIN_FP,
      O_ETRG_OUT              => my_etrg,
      I_ANA_TRG               => TRIG,

      I_ADC_DCO               => ADC_DCO,
      I_ADC_FCO               => ADC_FCO,
      I_ADC_OUT(0)            => ADC_OUT1,
      I_ADC_OUT(1)            => ADC_OUT2,
      I_ADC_OUT(2)            => ADC_OUT3,
      I_ADC_OUT(3)            => ADC_OUT4,
      O_ADC_CLK               => ADC_CLK,

      O_DRS_DENABLE           => DRS_DENABLE,
      O_DRS_DWRITEx           => DRS_DWRITE,
      O_DRS_SRINx             => DRS_SRIN,
      O_DRS_SRCLKx            => DRS_SRCLK,
      O_DRS_RSRLOADx          => DRS_RSRLOAD,
      O_DRS_A                 => DRS_A,

      I_DRS_WSROUTx(0)          => DRS1_WSROUT,
      I_DRS_SROUTx(0)           => DRS1_SROUT,
      I_DRS_PLLLCKx(0)          => DRS1_PLLLCK,

      I_DRS_WSROUTx(1)          => DRS2_WSROUT,
      I_DRS_SROUTx(1)           => DRS2_SROUT,
      I_DRS_PLLLCKx(1)          => DRS2_PLLLCK,

      I_DRS_WSROUTx(2)          => DRS3_WSROUT,
      I_DRS_SROUTx(2)           => DRS3_SROUT,
      I_DRS_PLLLCKx(2)          => DRS3_PLLLCK,

      I_DRS_WSROUTx(3)          => DRS4_WSROUT,
      I_DRS_SROUTx(3)           => DRS4_SROUT,
      I_DRS_PLLLCKx(3)          => DRS4_PLLLCK,


      I_TS_SDO                => TS_SDO,
      O_TS_SDI                => TS_SDI,
      O_TS_SCK                => TS_SCK,
      O_TS_CS(0)              => TS1CS,
      O_TS_CS(1)              => TS2CS,
      O_TS_CS(2)              => TS3CS,
      O_TS_CS(3)              => TS4CS,

      O_DAC_CS                => DAC_CS,
      O_DAC_SCK               => DAC_SCK,
      O_DAC_SDI               => DAC_SDI,

      O_EEPROM_CS             => EECS,

      O_DPRAM_CLK             => dpram_clk,

      O_DPRAM_EEPROM_ADDR     => dpram_eeprom_addr,
      O_DPRAM_EEPROM_D_WR     => dpram_eeprom_d_wr,
      O_DPRAM_EEPROM_WE       => dpram_eeprom_we,
      I_DPRAM_EEPROM_D_RD     => dpram_eeprom_d_rd,

      O_DPRAM_ADDR            => dpram_addr,
      O_DPRAM_D_WR            => dpram_d_wr,
      O_DPRAM_WE              => dpram_we,
      I_DPRAM_D_RD            => dpram_d_rd,

      I_CONTROL_REG_ARR       => control_reg_arr,
      O_STATUS_REG_ARR        => status_reg_arr,

      I_BYTESEL_ARR           => bytesel_arr,
      I_BITSET_VEC            => bitset_vec

      );


-- clock division
--clk_division: process(clk_pllout)
--begin
--  if (rising_edge(clk_pllout)) then
--    clk_cnt <= clk_cnt + 1;
--  end if;
--end process clk_division;

-- array of saved locbus addresses because of dpram read delay
-- index 0 is newest
saved_locbus_addr: process(global_reset, dpram_clk)
begin
  if(global_reset='1') then
    locbus_addr_saved <= (others => X"00000000");
  elsif (rising_edge(dpram_clk)) then
    loop_save_locbus_addr: for i in 0 to (n_locbus_addr_saved-2) loop
      locbus_addr_saved(i+1) <= locbus_addr_saved(i);  
    end loop loop_save_locbus_addr;
  end if;
  locbus_addr_saved(0) <= locbus_addr;  
end process saved_locbus_addr;

-- combinational ---------------------------------------------------------------
logic0        <= '0';

-- LEDs 
-- 0 1
-- 2 3
-- 4 5
-- 6 7

led_ready   <= x_ready_d;                        -- LED0: uC up and PLL locked
led_acquire <=     status_reg_arr(1)(4)          -- LED1: denable and dwrite
               and status_reg_arr(1)(5);         --       chip acquiring data
led_busy    <= status_reg_arr(1)(9);             -- LED2: busy 
led_error   <= status_reg_arr(1)(10);            -- LED3: running and waiting for trigger
led_trigin  <= my_etrg;                          -- LED4: trigger as recognized by FPGA trigger logic 
led_datatx  <= NOT(my_slrdn AND my_slwrn);       -- LED5: moving data to/from uC 
led_tcal    <= tca_ctrl AND NOT(vcal);           -- LED6: timing calibration with internally generated 100 Mhz signal 
led_fwupd   <= '0';

BOARDID_inv <= NOT BOARDID;  -- inversion needed to reflect correct enconding (schematics simplification)

-- DEBUG: to see anything recognizable in the status registers
-- preset_status_reg_debug: for i in type_status_reg_no'low to type_status_reg_no'high generate
--  status_reg_arr(i) <= X"BABE" & std_logic_vector(to_unsigned(i,16));
--end generate preset_status_reg_debug;

global_reset <= NOT pll_locked;
UC_RESETn    <= pll_locked;
INTn         <= 'Z';

-- uC
CTL6   <= 'Z';
CTL8   <= 'Z';
CTL9   <= 'Z';
CTL10  <= 'Z';
GPIO50 <= 'Z';
GPIO51 <= 'Z';
SLRDn  <= my_slrdn;
SLWRn  <= my_slwrn;
ucflag <= CTL8 & CTL6 & FLAGB & FLAGA;

-- DRS
RESETOUT    <= 'Z';    -- DRS4 uses internal reset
DRS_WSRIN   <= '1';    -- not used, no cascading/daisy-chaining
SEL_CAL <= vcal;
SEL_IN  <= NOT (vcal OR tca_ctrl);
-- SEL_TC  <= '1';        -- On-board Crystal Oscillator
-- SEL_TC  <= '0';        -- External timing calibration signal (J3 Connector)
SEL_TC  <= NOT tcal_sel;
CXOE    <= tca_ctrl;
VCALDIS <= NOT vcal;
TCAE    <= NOT tca_ctrl;
BUSY_FP     <= '0';
STAT        <= (others => '0');

-- DPRAM selection
locbus_eeprom_we   <= locbus_we when ((locbus_addr >= X"00020000") and (locbus_addr <= X"0002FFFF")) else '0';
locbus_dpram_we(0) <= locbus_we when ((locbus_addr >= X"00040000") and (locbus_addr <= X"00043FFF")) else '0';
locbus_dpram_we(1) <= locbus_we when ((locbus_addr >= X"00044000") and (locbus_addr <= X"00047FFF")) else '0';
locbus_dpram_we(2) <= locbus_we when ((locbus_addr >= X"00048000") and (locbus_addr <= X"0004BFFF")) else '0';
locbus_dpram_we(3) <= locbus_we when ((locbus_addr >= X"0004c000") and (locbus_addr <= X"0004FFFF")) else '0';
locbus_dpram_we(4) <= locbus_we when ((locbus_addr >= X"00080000") and (locbus_addr <= X"00083FFF")) else '0';
locbus_dpram_we(5) <= locbus_we when ((locbus_addr >= X"00084000") and (locbus_addr <= X"00087FFF")) else '0';
locbus_dpram_we(6) <= locbus_we when ((locbus_addr >= X"00088000") and (locbus_addr <= X"0008BFFF")) else '0';
locbus_dpram_we(7) <= locbus_we when ((locbus_addr >= X"0008C000") and (locbus_addr <= X"0008FFFF")) else '0';


-- use locbus_addr from previous dpram clock cycle
locbus_d_rd <= q_a_locbus_eeprom     when ((locbus_addr_saved(1) >= X"00020000") and (locbus_addr_saved(1) <= X"0002FFFF")) else
               q_a_locbus_readout(0) when ((locbus_addr_saved(1) >= X"00040000") and (locbus_addr_saved(1) <= X"00043FFF")) else
               q_a_locbus_readout(1) when ((locbus_addr_saved(1) >= X"00044000") and (locbus_addr_saved(1) <= X"00047FFF")) else
               q_a_locbus_readout(2) when ((locbus_addr_saved(1) >= X"00048000") and (locbus_addr_saved(1) <= X"0004BFFF")) else
               q_a_locbus_readout(3) when ((locbus_addr_saved(1) >= X"0004C000") and (locbus_addr_saved(1) <= X"0004FFFF")) else
               q_a_locbus_readout(4) when ((locbus_addr_saved(1) >= X"00080000") and (locbus_addr_saved(1) <= X"00083FFF")) else
               q_a_locbus_readout(5) when ((locbus_addr_saved(1) >= X"00084000") and (locbus_addr_saved(1) <= X"00087FFF")) else
               q_a_locbus_readout(6) when ((locbus_addr_saved(1) >= X"00088000") and (locbus_addr_saved(1) <= X"0008BFFF")) else
               q_a_locbus_readout(7) when ((locbus_addr_saved(1) >= X"0008C000") and (locbus_addr_saved(1) <= X"0008FFFF")) else
               X"C0FFEE00";

-- DDL
DDL_DATOUT  <= '0';
DDL_CLKOUT  <= '0';
DDL_EN      <= '0';

UTXEN       <= '0';
UTX         <= (others => '0');
URXEN       <= '0';

TESTIO      <= (others => 'Z');

UC_UART_CTSn <= 'Z';
UC_UART_RX   <= 'Z';

end struct;
