-- Inherited from S. Ritt et al.
-- Modified by S. Debieux
-- 20160912 further modified by Wolfgang.Rauch@cern.ch

library ieee;
use ieee.std_logic_1164.all;
-- use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
use ieee.numeric_std.all;
-- synopsys translate_off
library UNISIM;
use UNISIM.Vcomponents.ALL;
-- synopsys translate_on
use work.dpnc342_pack.all;

entity drs4_eval5_app is
  port (
    -- clocks
    I_DRS_READOUT_CLK       : in std_logic;
--    I_CLK66                 : in std_logic;
    I_RESET                 : in std_logic; -- active high power-up reset

    -- calibration
    O_CAL                   : out std_logic;
    O_TCA_CTRL              : out std_logic;
    O_TCA_SEL               : out std_logic;

    -- analog triggers
    I_ANA_TRG               : in std_logic_vector(3 downto 0);
    
    -- external trigger
    I_ETRG_IN               : in  std_logic;
    O_ETRG_OUT              : out std_logic;

    I_ADC_DCO               : in  std_logic;
    I_ADC_FCO               : in  std_logic;
    I_ADC_OUT               : in  std_logic_vector(3 downto 0);
    O_ADC_CLK               : out std_logic;

    O_DRS_DENABLE           : out std_logic;
    O_DRS_DWRITEx           : out std_logic;
    O_DRS_SRINx             : out std_logic;
    O_DRS_SRCLKx            : out std_logic;
    O_DRS_RSRLOADx          : out std_logic;
    O_DRS_A                 : out std_logic_vector(3 downto 0);

    I_DRS_WSROUTx           : in  std_logic_vector(3 downto 0);
    I_DRS_SROUTx            : in  std_logic_vector(3 downto 0);
    I_DRS_PLLLCKx           : in  std_logic_vector(3 downto 0);

    I_TS_SDO                : in  std_logic;
    O_TS_SDI                : out std_logic;
    O_TS_SCK                : out std_logic;
    O_TS_CS                 : out std_logic_vector(3 downto 0);

    O_DAC_CS                : out std_logic;
    O_DAC_SCK               : out std_logic;
    O_DAC_SDI               : out std_logic;
    
    O_EEPROM_CS             : out std_logic;

    -- Simple bus interface to DPRAM

    O_DPRAM_CLK        : out std_logic;

    O_DPRAM_EEPROM_ADDR       : out std_logic_vector((n_addr_bits_w32_dpram_eeprom-1+2) downto 0);
    O_DPRAM_EEPROM_D_WR       : out std_logic_vector(31 downto 0);
    O_DPRAM_EEPROM_WE         : out std_logic;
    I_DPRAM_EEPROM_D_RD       : in std_logic_vector(31 downto 0);

    O_DPRAM_ADDR       : out t_dpram_readout_addr_a(7 downto 0);
    O_DPRAM_D_WR       : out t_dpram_readout_d_wr_a(7 downto 0);
    O_DPRAM_WE         : out std_logic_vector(7 downto 0);
    I_DPRAM_D_RD       : in t_dpram_readout_d_rd_a(7 downto 0);

    -- Control & status registers from system FPGA interface
    I_CONTROL_REG_ARR     : in  t_reg_ctrl_a;
    O_STATUS_REG_ARR      : out t_reg_stat_a;
    I_BYTESEL_ARR         : in  t_bytesel_a;
    I_BITSET_VEC          : in  std_logic_vector(t_bitset_b'pos(t_bitset_b'right) downto 0)

  );
end drs4_eval5_app;

--*************************************************************

architecture arch of drs4_eval5_app is

component ibufreg
  generic (
    width : integer := 1
  );
  port (
    CLK  : in  std_logic;
    PRE  : in  std_logic;
    IPIN : in  std_logic_vector(width-1 downto 0);
    O    : out std_logic_vector(width-1 downto 0)
  );
end component;

component ibufreg1
  port (
    CLK  : in  std_logic;
    PRE  : in  std_logic;
    IPIN : in  std_logic;
    O    : out std_logic
  );
end component;

component obufreg
  generic (
    width : integer := 1
  );
  port (
    CLK  : in  std_logic;
    PRE  : in  std_logic;
    I    : in  std_logic_vector(width-1 downto 0);
    OPIN : out std_logic_vector(width-1 downto 0)
  );
end component;

component obufreg1
  port (
    CLK  : in  std_logic;
    PRE  : in  std_logic;
    I    : in  std_logic;
    OPIN : out std_logic
  );
end component;

component deserf12_a
  generic (
    width : integer := 1
  );
  port (
    CLK    : in  std_logic;
    DCO    : in  std_logic;
    FCO    : in  std_logic;
    PRE    : in  std_logic;
    D      : in  std_logic_vector((width-1) downto 0);
    Q      : out t_drs_adc_val_a((width-1) downto 0)
  );
end component;

  type t_drs_stop_cell is array (3 downto 0) of std_logic_vector(9 downto 0);
  type t_drs_stop_wsr  is array (3 downto 0) of std_logic_vector(7 downto 0);

  signal GND  : std_logic;
  signal VCC  : std_logic;

  -- ADC
  signal i_drs_adc               : t_drs_adc_val_a(3 downto 0);
  signal o_drs_adc_clk           : std_logic;
  signal adc_clk_sr              : std_logic_vector(15 downto 0);
  -- Serial interface for DAC, EEPROM and Temp. Sensor
  signal o_drs_serial_data       : std_logic;
  signal o_drs_serial_clk        : std_logic;
  signal o_drs_ts_sck            : std_logic;
  signal i_drs_serial_data       : std_logic;
  signal i_drs_eeprom_data       : std_logic;
  signal o_drs_dac_cs_n          : std_logic;
  signal o_drs_eeprom_cs_n       : std_logic;
  signal o_drs_ts_cs_n           : std_logic_vector(3 downto 0);
  -- DRS Start/enable
  signal o_drs_enable            : std_logic;
  signal o_drs_write             : std_logic;
  -- Internal DRS shift registers
  signal o_drs_srin              : std_logic;
  signal o_drs_srclk             : std_logic;
  signal o_drs_rsrload           : std_logic;
  signal i_drs_srout             : std_logic_vector(3 downto 0);
  signal i_drs_wsrout            : std_logic_vector(3 downto 0);
  subtype t_sr_count is integer range 0 to 1024;
  signal drs_sr_count            : t_sr_count;
  signal drs_sr_reg              : std_logic_vector(7 downto 0);
  -- DRS address
  signal o_drs_addr              : std_logic_vector(3 downto 0);
  -- PLL refence clock signal
  signal drs_refclk              : std_logic;
--  signal o_drs_refclk            : std_logic;
  signal drs_refclk_counter      : std_logic_vector(16 downto 0);
  signal i_drs_plllck            : std_logic_vector(3 downto 0);
  -- 132/264 MHz calibration signal output
  signal o_drs_tcalib_sig        : std_logic;
  
  -- Control registers
  signal drs_ctl_start_trig      : std_logic;
  signal drs_ctl_reinit_trig     : std_logic;  -- 1 sets drs_reinit_reqest to '1'
  signal drs_ctl_soft_trig       : std_logic;
  signal drs_ctl_eeprom_write_trig: std_logic;
  signal drs_ctl_eeprom_read_trig: std_logic;
  signal drs_ctl_autostart       : std_logic;
  signal drs_ctl_dmode           : std_logic;
  signal drs_ctl_dactive         : std_logic;
  signal drs_ctl_adc_active      : std_logic;
  signal drs_ctl_acalib          : std_logic;
--  signal drs_ctl_led_yellow      : std_logic;
  signal drs_ctl_tca_ctrl        : std_logic;
--  signal drs_ctl_refclk_source   : std_logic;
  signal drs_ctl_tcal_source     : std_logic;
  signal drs_ctl_dac_arr         : t_drs_dac_val_a((n_drs_dacs-1) downto 0);
--  next signal foreseen as flag for first channel action 
--  if first_chn and last_chn together interpreted as 8-bit read-out mask
--  if this will ever happen :-)
--  signal drs_read_first_chn       : std_logic;
  signal drs_ctl_first_chn       : std_logic_vector(3 downto 0);
  signal drs_ctl_last_chn        : std_logic_vector(3 downto 0);
  signal drs_ctl_config          : std_logic_vector(7 downto 0);
  signal drs_ctl_chn_config      : std_logic_vector(7 downto 0);
  signal drs_ctl_sampling_freq   : std_logic_vector(15 downto 0);
  signal drs_ctl_standby_mode    : std_logic;
  signal drs_ctl_enable_trigger  : std_logic;
  signal drs_ctl_trigger_config  : std_logic_vector(15 downto 0);
  signal drs_ctl_neg_trigger     : std_logic;
  signal drs_ctl_readout_mode    : std_logic;
  signal drs_ctl_delay_sel       : std_logic_vector(7 downto 0);

  -- Status registers
  signal drs_stat_busy           : std_logic;
  signal drs_stat_idle           : std_logic;
  signal drs_stat_running        : std_logic;
  signal drs_eeprom_busy         : std_logic;
  signal drs_stat_stop_cell      : t_drs_stop_cell;
  signal drs_stat_stop_wsr       : t_drs_stop_wsr;
  signal drs_temperature         : t_drs_dac_val_a(3 downto 0);
  signal drs_trigger_bus         : std_logic_vector(15 downto 0);
  signal drs_serial_number       : std_logic_vector(15 downto 0);

  -- Misc. internal signals
  signal drs_reinit_request      : std_logic;
  signal drs_old_readout_mode    : std_logic;

  -- Trigger signals
  signal drs_trigger             : std_logic;
  signal drs_soft_trig           : std_logic;
  signal drs_trigger_syn         : std_logic;
  signal drs_write_set           : std_logic;
  signal drs_trig_ff             : std_logic;
  signal drs_write_ff            : std_logic;
  signal drs_hard_inp            : std_logic_vector(4 downto 0);
  signal drs_hard_or             : std_logic;
  signal drs_hard_and            : std_logic;
  signal drs_hard_trig           : std_logic;
  signal drs_arm_trig            : std_logic;
--  signal drs_trg_delay           : std_logic_vector(2047 downto 0);
--  -- Tell P&R to not optimize away the drs_trg_delay array
--  attribute keep : string;
--  attribute keep of drs_trg_delay : signal is "true";

  -- Serial bus internal signals
  type t_serial_bus_state is (idle, wait_serdes, eeprom_read, eeprom_write, done);
  signal serial_bus_state        : t_serial_bus_state;
  subtype t_serial_count is integer range 0 to 200;
  signal serial_count            : t_serial_count;
  signal serial_ret_addr         : t_serial_count;
  signal serial_start_flag1      : std_logic;
  signal serial_start_flag2      : std_logic;
  signal stat_serial_busy        : std_logic;

  type t_serdes_state is (idle, busy, busy_temp);
  signal serdes_state            : t_serdes_state;
  subtype t_serdes_clk is integer range 0 to 10;
  signal serdes_clk              : t_serdes_clk;
  signal serdes_speed            : t_serdes_clk;
  subtype t_serdes_count is integer range 0 to 100;
  signal serdes_count            : t_serdes_count;
  subtype t_serdes_bit_count_m1 is integer range 0 to 32;
  signal serdes_bit_count_m1        : t_serdes_bit_count_m1;
  signal serdes_bit_no           : t_serdes_bit_count_m1;
  signal serdes_trig             : std_logic;
  signal serdes_trig_temp        : std_logic;
  signal serdes_wdata            : std_logic_vector(31 downto 0);
  signal serdes_rdata            : std_logic_vector(31 downto 0);
  signal stat_serdes_busy        : std_logic;
  signal stat_eeprom_spi_command : std_logic_vector(7 downto 0);  -- diagnostic
  signal stat_eeprom_sc_debug    : std_logic_vector(7 downto 0);  -- diagnostic

  type  t_drs_dac_reg_a is array ((n_drs_dacs-1) downto 0) of std_logic_vector(15 downto 0);
  signal drs_dac_reg             : t_drs_dac_reg_a;
  signal drs_dac_newval_flag     : std_logic_vector((n_drs_dacs -1) downto 0); 

  signal temp                    : std_logic_vector(15 downto 0);
  signal temp_timer              : std_logic_vector(25 downto 0); -- once per second
  signal temp_cmd                : std_logic_vector(7 downto 0);

  subtype t_eeprom_count is integer range 0 to 100;
  signal drs_eeprom_write_trig   : std_logic;
  signal drs_eeprom_read_trig    : std_logic;
  signal drs_ctl_eeprom_sector   : std_logic_vector(15 downto 0);
  signal drs_eeprom_sector       : std_logic_vector(7 downto 0);
  signal drs_eeprom_page         : std_logic_vector(7 downto 0);  
  signal drs_eeprom_byte         : std_logic_vector(7 downto 0);
  signal drs_eeprom_cmd          : std_logic_vector(59 downto 0);

  -- state of DRS readout state machine
  type t_drs_readout_state is (init, idle, done, trailer,
                                  start_running, running, start_readout, wait_vdd, adc_sync,
                                  chip_readout, wsr_addr, wsr_setup, wsr_strobe, conf_setup, 
                                  conf_strobe, init_rsr, waitforfifo, fifo1transfer, fifo2transfer,
                                  fifo3transfer, fifo4transfer);
  signal drs_readout_state        : t_drs_readout_state;
  -- Localy copy signal to fulfull timing constraings
  signal drs_readout_state_local  : t_drs_readout_state;

  -- Clock signals for freq. meas., DRS data readout and serial bus (see below)
  signal drs_readout_clk          : std_logic;  -- clock for DRS readout state machine
  signal drs_readout_clk_ps       : std_logic;  -- clock for FADC, phase shifted
  signal drs_serial_clk           : std_logic;  -- clock for serial bus

  -- Signals for DRS waveform readout
  subtype t_drs_start_timer is integer range 0 to 255;
  signal drs_start_timer          : t_drs_start_timer; 
  signal drs_sample_count         : std_logic_vector(10 downto 0);

-- Wolfgang.Rauch@cern.ch adjusted and renamed for 66 MHz clock
--  signal drs_vdd_count            : std_logic_vector(12 downto 0);
  signal drs_vdd_count            : std_logic_vector(13 downto 0);

  signal drs_rd_tmp_count         : std_logic_vector(12 downto 0);

  signal drs_stop_cell            : t_drs_stop_cell;
  signal drs_stop_wsr             : t_drs_stop_wsr;
  signal drs_addr                 : std_logic_vector(3 downto 0);

  -- DPRAM interface signals
  signal dpram_readout_reset     : std_logic;
  signal dpram_readout_we        : std_logic_vector(3 downto 0);
  signal dpram_readout_addr      : t_dpram_readout_addr_a(3 downto 0);
  signal dpram_readout_addr_next : t_dpram_readout_addr_a(3 downto 0);
  signal dpram_readout_d_rd      : t_dpram_readout_d_rd_a(3 downto 0);
  signal dpram_readout_d_wr      : t_dpram_readout_d_wr_a(3 downto 0);
  signal dpram_pedest_reset      : std_logic;
  signal dpram_pedest_we         : std_logic_vector(3 downto 0);
  signal dpram_pedest_addr       : t_dpram_readout_addr_a(3 downto 0);
  signal dpram_pedest_addr_next  : t_dpram_readout_addr_a(3 downto 0);
  signal dpram_pedest_d_wr       : t_dpram_readout_d_wr_a(3 downto 0);
  signal dpram_pedest_d_rd       : t_dpram_readout_d_rd_a(3 downto 0);
  signal dpram_eeprom_reset      : std_logic;
  signal dpram_eeprom_we         : std_logic;
  signal dpram_eeprom_addr       : std_logic_vector((n_addr_bits_w32_dpram_eeprom-1)+2 downto 0);
  signal dpram_eeprom_addr_next  : std_logic_vector((n_addr_bits_w32_dpram_eeprom-1)+2 downto 0);
  signal dpram_eeprom_d_wr       : std_logic_vector(31 downto 0);
  signal dpram_eeprom_d_rd       : std_logic_vector(31 downto 0);
--  signal dpram_eeprom_inc        : std_logic;

  -- Frequency counter etc.
  subtype t_32_bit is std_logic_vector(31 downto 0);
  type t_scaler_a is array (5 downto 0) of t_32_bit;
  
  signal drs_1hz_counter          : std_logic_vector(31 downto 0);
  signal drs_1hz_clock            : std_logic;
  signal drs_1hz_latch1           : std_logic_vector(5 downto 0);
  signal drs_1hz_latch2           : std_logic_vector(5 downto 0);
  signal drs_1hz_latch3           : std_logic_vector(5 downto 0);
 
  signal scaler                   : t_scaler_a;
  signal scaler_clock             : std_logic_vector(5 downto 0);
  signal scaler_reset             : std_logic_vector(5 downto 0);
  signal scaler_ff                : std_logic_vector(5 downto 0);
  signal scaler_ff_reset          : std_logic_vector(5 downto 0);

  signal l1_fifo_wr  : std_logic;
  signal l1_fifo_rd1 : std_logic;
  signal l1_fifo_rd2 : std_logic;
  signal l1_fifo_rd3 : std_logic;
  signal l1_fifo_rd4 : std_logic;

begin
  GND   <= '0';
  VCC   <= '1';

  -- Clock signals for freq. meas., DRS data readout and DAC control
  drs_readout_clk       <= I_DRS_READOUT_CLK;
  drs_readout_clk_ps    <= I_DRS_READOUT_CLK; -- phase shifted clock for FADC
  drs_serial_clk        <= I_DRS_READOUT_CLK;

  ibuf_drs_sdo : ibufreg1
  port map (
    CLK  => drs_serial_clk,
    PRE  => I_RESET,
    IPIN => I_TS_SDO,
    O    => i_drs_serial_data
  );

--  ibuf_drs_sdo_eeprom : ibufreg1
--  port map (
--    CLK  => drs_serial_clk,
--    PRE  => I_RESET,
--    IPIN => I_TS_SDO,
--    O    => i_drs_eeprom_data
--  );
  i_drs_eeprom_data <=  i_drs_serial_data;

  obuf_drs_sdi : obufreg1
  port map (
    CLK  => drs_serial_clk,
    PRE  => I_RESET,
    I    => o_drs_serial_data,
    OPIN => O_TS_SDI           -- not needed for temperature sensors, needed for EEPROM
  );

  obuf_drs_sck : obufreg1
  port map (
    CLK  => drs_serial_clk,
    PRE  => I_RESET,
    I    => o_drs_ts_sck,
    OPIN => O_TS_SCK
  );
  
  obuf_drs_ts_cs_a : obufreg
  generic map (
    width => 4
  )
  port map (
    CLK  => drs_serial_clk,
    PRE  => I_RESET,
    I    => o_drs_ts_cs_n,
    OPIN => O_TS_CS
  );

  obuf_dac_cs : obufreg1
  port map (
    CLK  => drs_serial_clk,
    PRE  => I_RESET,
    I    => o_drs_dac_cs_n,
    OPIN => O_DAC_CS
  );

  obuf_eeprom_cs : obufreg1
  port map (
    CLK  => drs_serial_clk,
    PRE  => I_RESET,
    I    => o_drs_eeprom_cs_n,
    OPIN => O_EEPROM_CS
  );

  obuf_dac_sck : obufreg1
  port map (
    CLK  => drs_serial_clk,
    PRE  => I_RESET,
    I    => o_drs_serial_clk,
    OPIN => O_DAC_SCK
  );

  obuf_dac_sdi : obufreg1
  port map (
    CLK  => drs_serial_clk,
    PRE  => I_RESET,
    I    => o_drs_serial_data,
    OPIN => O_DAC_SDI
  );

  obuf_drs_denable : obufreg1
  port map (
    CLK  => drs_readout_clk,
    PRE  => I_RESET,
    I    => o_drs_enable,
    OPIN => O_DRS_DENABLE
  );

  ibuf_drs_wsrout : ibufreg
  generic map(
    width => 4
  )
  port map (
    CLK  => drs_readout_clk,
    PRE  => I_RESET,
    IPIN => I_DRS_WSROUTx,
    O    => i_drs_wsrout
  );

O_DRS_DWRITEx <= o_drs_write;

  obuf_drs_srin : obufreg1
  port map (
    CLK  => drs_readout_clk,
    PRE  => I_RESET,
    I    => o_drs_srin,
    OPIN => O_DRS_SRINx
  );

  obuf_drs_srclk : obufreg1
  port map (
    CLK  => drs_readout_clk,
    PRE  => I_RESET,
    I    => o_drs_srclk,
    OPIN => O_DRS_SRCLKx
  );

  ibuf_drs_srout : ibufreg
  generic map(
    width => 4
  )
  port map (
    CLK  => drs_readout_clk,
    PRE  => I_RESET,
    IPIN => I_DRS_SROUTx,
    O    => i_drs_srout
  );

  obuf_drs_rsrload : obufreg1
  port map (
    CLK  => drs_readout_clk,
    PRE  => I_RESET,
    I    => o_drs_rsrload,
    OPIN => O_DRS_RSRLOADx
  );

  ibuf_drs_plllck : ibufreg
  generic map(
    width => 4
  )
  port map (
    CLK  => drs_readout_clk,
    PRE  => I_RESET,
    IPIN => I_DRS_PLLLCKx,
    O    => i_drs_plllck
  );

  obuf_drs_a : obufreg
  generic map (
    width => 4
  )
  port map (
    CLK  => drs_readout_clk,
    PRE  => I_RESET,
    I    => o_drs_addr,
    OPIN => O_DRS_A
  );

  adc_data : deserf12_a
  generic map (
    width => 4
  )
  port map (
    CLK    => o_drs_adc_clk,
    DCO    => I_ADC_DCO,
    FCO    => I_ADC_FCO,
    PRE    => I_RESET,
    D      => I_ADC_OUT,
    Q      => i_drs_adc
  );

O_ADC_CLK <= o_drs_adc_clk;

  -- UsrBus control register mapping (control registers are set/read via
  -- UsrBus/System FPGA). In addition to the normal control register bits
  -- I_CONTROL_REG_ARR(reg_no)(bit_no), control register 0 has a special
  -- feature: a bit in the std_logic_vector I_BITSET_VEC(0)(2 downto 0) is
  -- set for one I_DRS_READOUT_CLK cycle (normally it is '0') when the System FPGA
  -- writes a '1' to the respective bit in control register 0. This can
  -- be used to trigger an event with a single UsrBus write cycle (instead
  -- of toggling the control register bit 0->1->0 using two write cycles).
  -- Wolfgang.Rauch@cern.ch added:
  -- EEPROM read/write bits moved to different register
  drs_ctl_start_trig        <= I_BITSET_VEC(t_bitset_b'pos(bitset_start));
  drs_ctl_reinit_trig       <= I_BITSET_VEC(t_bitset_b'pos(bitset_reinit));
  drs_ctl_soft_trig         <= I_BITSET_VEC(t_bitset_b'pos(bitset_soft));
  drs_ctl_eeprom_write_trig <= I_BITSET_VEC(t_bitset_b'pos(bitset_eeprom_w));
  drs_ctl_eeprom_read_trig  <= I_BITSET_VEC(t_bitset_b'pos(bitset_eeprom_r));
  drs_ctl_autostart         <= I_CONTROL_REG_ARR(1)(0);
  drs_ctl_adc_active        <= I_CONTROL_REG_ARR(1)(1);
  drs_ctl_tca_ctrl          <= I_CONTROL_REG_ARR(1)(3);
  drs_ctl_enable_trigger    <= I_CONTROL_REG_ARR(1)(6);
  drs_ctl_readout_mode      <= I_CONTROL_REG_ARR(1)(7);
  drs_ctl_neg_trigger       <= I_CONTROL_REG_ARR(1)(8);
  drs_ctl_acalib            <= I_CONTROL_REG_ARR(1)(9);
--  drs_ctl_refclk_source     <= I_CONTROL_REG_ARR(1)(10);
  drs_ctl_tcal_source       <= I_CONTROL_REG_ARR(1)(10);
  drs_ctl_dactive           <= I_CONTROL_REG_ARR(1)(11);
  drs_ctl_standby_mode      <= I_CONTROL_REG_ARR(1)(12);
  
  drs_ctl_dac_arr(0)        <= I_CONTROL_REG_ARR(2)(15 downto 0);  -- needs byte select.
  drs_ctl_dac_arr(1)        <= I_CONTROL_REG_ARR(3)(15 downto 0);  -- needs byte select.
  drs_ctl_dac_arr(2)        <= I_CONTROL_REG_ARR(4)(15 downto 0);  -- needs byte select.
  drs_ctl_dac_arr(3)        <= I_CONTROL_REG_ARR(5)(15 downto 0);  -- needs byte select.
  drs_ctl_dac_arr(4)        <= I_CONTROL_REG_ARR(6)(15 downto 0);  -- needs byte select.
-- drs_ctl_dac_arr(5)        <= I_CONTROL_REG_ARR(7)(1 downto 0);    -- needs byte select.
-- drs_ctl_dac_arr(6)        <= I_CONTROL_REG_ARR(8)(1 downto 0);    -- needs byte select.
-- drs_ctl_dac_arr(7)        <= I_CONTROL_REG_ARR(9)(1 downto 0);    -- needs byte select.
  drs_ctl_first_chn         <= I_CONTROL_REG_ARR(10)(7 downto 4);
  drs_ctl_last_chn          <= I_CONTROL_REG_ARR(10)(3 downto 0);
  drs_ctl_chn_config        <= I_CONTROL_REG_ARR(10)(23 downto 16);  -- needs byte select.
  drs_ctl_config            <= I_CONTROL_REG_ARR(10)(31 downto 24);  -- needs byte select.
  drs_ctl_sampling_freq     <= I_CONTROL_REG_ARR(11)(15 downto 0);
  drs_ctl_delay_sel         <= I_CONTROL_REG_ARR(11)(23 downto 16);
  drs_ctl_trigger_config    <= I_CONTROL_REG_ARR(12)(31 downto 16);
  drs_ctl_eeprom_sector     <= I_CONTROL_REG_ARR(13)(15 downto 0);
  
  drs_ctl_dmode             <= drs_ctl_config(0);
  
  -- UsrBus status register mapping
  O_STATUS_REG_ARR(0)( 7 downto  0) <= X"04";    -- DRS4
  O_STATUS_REG_ARR(0)(15 downto  8) <= X"09";    -- USB Evaluation Board 5.0
  O_STATUS_REG_ARR(0)(31 downto 16) <= X"C0DE";  -- magic number to identify PSI VPC board

  O_STATUS_REG_ARR(1)( 3 downto  0) <= i_drs_plllck;      -- DRS 1 - 4
  O_STATUS_REG_ARR(1)( 4)           <= o_drs_enable;      
  O_STATUS_REG_ARR(1)( 5)           <= o_drs_write;      
  O_STATUS_REG_ARR(1)( 6)           <= drs_arm_trig;      
  O_STATUS_REG_ARR(1)( 7)           <= drs_trig_ff;      
  O_STATUS_REG_ARR(1)( 8)           <= drs_soft_trig;      
  O_STATUS_REG_ARR(1)( 9)           <= drs_stat_busy;      
  O_STATUS_REG_ARR(1)(10)           <= drs_stat_running;      
  O_STATUS_REG_ARR(1)(11)           <= drs_stat_idle;      
  O_STATUS_REG_ARR(1)(28 downto 12) <= (others => '0');
  O_STATUS_REG_ARR(1)(29)           <= stat_serdes_busy;      
  O_STATUS_REG_ARR(1)(30)           <= stat_serial_busy;      
  O_STATUS_REG_ARR(1)(31)           <= drs_eeprom_busy;
  
  O_STATUS_REG_ARR(2)( 9 downto  0) <= drs_stat_stop_cell(0);
  O_STATUS_REG_ARR(2)(15 downto 10) <= (others => '0');
  O_STATUS_REG_ARR(2)(23 downto 16) <= drs_stat_stop_wsr(0);
  O_STATUS_REG_ARR(2)(31 downto 24) <= (others => '0');
  
  O_STATUS_REG_ARR(3)( 9 downto  0) <= drs_stat_stop_cell(1);
  O_STATUS_REG_ARR(3)(15 downto 10) <= (others => '0');
  O_STATUS_REG_ARR(3)(23 downto 16) <= drs_stat_stop_wsr(1);
  O_STATUS_REG_ARR(3)(31 downto 24) <= (others => '0');
  
  O_STATUS_REG_ARR(4)( 9 downto  0) <= drs_stat_stop_cell(2);
  O_STATUS_REG_ARR(4)(15 downto 10) <= (others => '0');
  O_STATUS_REG_ARR(4)(23 downto 16) <= drs_stat_stop_wsr(2);
  O_STATUS_REG_ARR(4)(31 downto 24) <= (others => '0');
  
  O_STATUS_REG_ARR(5)( 9 downto  0) <= drs_stat_stop_cell(3);
  O_STATUS_REG_ARR(5)(15 downto 10) <= (others => '0');
  O_STATUS_REG_ARR(5)(23 downto 16) <= drs_stat_stop_wsr(3);
  O_STATUS_REG_ARR(5)(31 downto 24) <= (others => '0');
  
  O_STATUS_REG_ARR(6)( 7 downto  0) <= stat_eeprom_spi_command; -- diagnostic
  O_STATUS_REG_ARR(6)(15 downto  8) <= stat_eeprom_sc_debug;    -- diagnostic
  O_STATUS_REG_ARR(6)(31 downto 16) <= drs_trigger_bus;

  O_STATUS_REG_ARR(7)(15 downto  0) <= drs_temperature(0);
  O_STATUS_REG_ARR(7)(31 downto 16) <= drs_temperature(1);
  
  O_STATUS_REG_ARR(8)(15 downto  0) <= drs_temperature(2);
  O_STATUS_REG_ARR(8)(31 downto 16) <= drs_temperature(3);

  O_STATUS_REG_ARR(9)(15 downto 0)  <= svn_revision;
--  O_STATUS_REG_ARR(9)(31 downto 16) <= board_sn;        -- hard-coded serial number in package, no EEPROM attached to FPGA
  O_STATUS_REG_ARR(9)(31 downto 16) <= drs_serial_number;

--  O_STATUS_REG_ARR(15 downto 10)    updated in proc_drs_scaler;

  O_DPRAM_CLK              <= drs_readout_clk;

-- Wolfgang.Rauch@cern.ch changed to private eeprom dpram 
  O_DPRAM_EEPROM_WE     <= dpram_eeprom_we;
  O_DPRAM_EEPROM_ADDR   <= dpram_eeprom_addr;
  O_DPRAM_EEPROM_D_WR   <= dpram_eeprom_d_wr;

-- DPRAM for waveform data and EEPROM read/write
  O_DPRAM_ADDR(3 downto 0) <= dpram_readout_addr;
  O_DPRAM_ADDR(7 downto 4) <= dpram_pedest_addr;
  O_DPRAM_WE(3 downto 0)   <= dpram_readout_we;
  O_DPRAM_WE(7 downto 4)   <= dpram_pedest_we;
  O_DPRAM_D_WR(3 downto 0) <= dpram_readout_d_wr;
  O_DPRAM_D_WR(7 downto 4) <= dpram_pedest_d_wr;

  dpram_eeprom_d_rd  <= I_DPRAM_EEPROM_D_RD;
  dpram_readout_d_rd <= I_DPRAM_D_RD(3 downto 0);                
  dpram_pedest_d_rd  <= I_DPRAM_D_RD(7 downto 4);                

-- Wolfgang.Rauch@cern.ch -- the EEPROM stuff is presently broken
  -- State machine for serial bus access. It contains four functions:
  -- 1) Set the 8 DAC channels whenever a write access to
  --    the respective 16-bit word in the respective control register
  --    occurs (via UsrBus/System FPGA, see above for register map)
  -- 2) Read temperature sensor once per second
  -- 3) Read EEPROM serial number on boot
  -- 4) Write EEPROM serial number on request

  proc_serial_io: process(I_RESET, drs_serial_clk)
  begin
    if (I_RESET = '1') then

      serial_bus_state         <= idle;
      stat_serial_busy         <= '0';
      stat_eeprom_spi_command  <= "00000000";  -- diagnostic

      drs_dac_newval_flag   <= (others => '0');
      drs_eeprom_write_trig <= '0';
      drs_eeprom_read_trig  <= '0';
--
-- Wolfgang.Rauch@cern.ch
-- for now we don't do an initial read
--      drs_eeprom_busy       <= '1'; -- will be cleard after initial read
      drs_eeprom_busy       <= '0';
      
      temp_timer            <= (others => '0');
      serial_count          <= 0;

      drs_dac_reg           <= (others => X"0000");

      serdes_trig           <= '0';
      serdes_trig_temp      <= '0';
      serdes_wdata          <= (others => '0');

      o_drs_dac_cs_n        <= '1';
      o_drs_eeprom_cs_n     <= '1';
      o_drs_ts_cs_n         <= (others => '1');
      serial_count          <= 0;
      
      serial_start_flag1    <= '1';
      serial_start_flag2    <= '1';
      
      dpram_eeprom_we         <= '0';
      dpram_eeprom_reset      <= '0';
      dpram_eeprom_addr       <= (others => '0');
      dpram_eeprom_addr_next  <= (others => '0');
      
    elsif rising_edge(drs_serial_clk) then
-- Wolfgang.Rauch@cern.ch
-- presently only temperature sensor 0 used
      o_drs_ts_cs_n(3 downto 1) <= (others => '1');

      stat_serial_busy        <= '1'; -- busy unless in idle state 
      dpram_eeprom_addr     <=  dpram_eeprom_addr_next;

      serial_count          <= serial_count + 1;
      serdes_trig           <= '0';
      serdes_trig_temp      <= '0';

      -- check if DAC control registers were updated (most significant byte triggers)
      gen_dac_newval_flag_set : for count in 0 to 4 loop  -- 5 DACs (index 0 to 4)
        if (I_BYTESEL_ARR(count+2)(3) = '1') then    -- 2 is index of first dac register
          drs_dac_newval_flag(count)      <= '1';
          drs_dac_reg(count)              <= drs_ctl_dac_arr(count);
        end if;
      end loop;

      -- check if eeprom write trigger was updated
      if (drs_ctl_eeprom_write_trig = '1') then
        drs_eeprom_write_trig   <= '1';
        drs_eeprom_busy         <= '1';
      end if;

      -- check if eeprom read trigger was updated
      if (drs_ctl_eeprom_read_trig = '1') then
        drs_eeprom_read_trig    <= '1';
        drs_eeprom_busy         <= '1';
      end if;

      case (serial_bus_state) is

        when idle =>
          stat_serial_busy      <= '0'; -- busy unless in idle state 
          o_drs_dac_cs_n        <= '1';
          o_drs_eeprom_cs_n     <= '1';
          o_drs_ts_cs_n(0)      <= '1';
          serial_count          <= 0;
          dpram_eeprom_we       <= '0';
          
          -- need if..elsif chain to get only one flag set
          if (drs_dac_newval_flag(0) = '1') then
            drs_dac_newval_flag(0)    <= '0';
            serdes_wdata(23 downto 0) <= "00110000" & drs_dac_reg(0);
          elsif (drs_dac_newval_flag(1) = '1') then
            drs_dac_newval_flag(1)    <= '0';
            serdes_wdata(23 downto 0) <= "00110001" & drs_dac_reg(1);
          elsif (drs_dac_newval_flag(2) = '1') then
            drs_dac_newval_flag(2)    <= '0';
            serdes_wdata(23 downto 0) <= "00110010" & drs_dac_reg(2);
          elsif (drs_dac_newval_flag(3) = '1') then
            drs_dac_newval_flag(3)    <= '0';
            serdes_wdata(23 downto 0) <= "00110011" & drs_dac_reg(3);
          elsif (drs_dac_newval_flag(4) = '1') then
            drs_dac_newval_flag(4)    <= '0';
            serdes_wdata(23 downto 0) <= "00110100" & drs_dac_reg(4);
--          elsif (drs_dac_newval_flag(5) = '1') then
--            drs_dac_newval_flag(5)    <= '0';
--            serdes_wdata(23 downto 0) <= "00110101" & drs_dac_reg(5);
--          elsif (drs_dac_newval_flag(6) = '1') then
--            drs_dac_newval_flag(6)    <= '0';
--            serdes_wdata(23 downto 0) <= "00110110" & drs_dac_reg(6);
--          elsif (drs_dac_newval_flag(7) = '1') then
--            drs_dac_newval_flag(7)    <= '0';
--            serdes_wdata(23 downto 0) <= "00110111" & drs_dac_reg(7);
          end if;

          -- start serdes process if DAC changed
          if (drs_dac_newval_flag /= "00000") then
            serial_bus_state         <= wait_serdes;
            o_drs_dac_cs_n           <= '0';
            serdes_bit_count_m1      <= 23;
            serdes_speed             <= 0;
            serdes_trig              <= '1';
          end if;

          -- read eeprom once after startup
          -- need delay such that o_drs_eeprom_cs_n = 1 for long enough time

-- Wolfgang.Rauch@cern.ch: presently next four lines do hurt
--          if (serial_start_flag1 = '1' and temp_timer(12) = '1') then
--            serial_start_flag1       <= '0';
--            serial_bus_state         <= eeprom_read;
--          end if;

-- Wolfgang.Rauch@cern.ch: this has to be adjusted for the other three temperature sensors
          -- read temperature when timer expired
          if (temp_timer(temp_timer'high) = '1' or serial_start_flag2 = '1') then
            serial_start_flag2       <= '0';
            temp_timer               <= (others => '0');
            serial_bus_state         <= wait_serdes;
            o_drs_ts_cs_n(0)         <= '0';
            serdes_trig_temp         <= '1';
          else
            temp_timer               <= temp_timer + "1";
          end if;

          -- write EEPROM when triggered
          if (drs_eeprom_write_trig = '1') then
            drs_eeprom_write_trig    <= '0';
            serial_bus_state         <= eeprom_write;
            drs_eeprom_sector        <= drs_ctl_eeprom_sector(7 downto 0);
          end if;

          -- read EEPROM when triggered
          if (drs_eeprom_read_trig = '1') then
            drs_eeprom_read_trig     <= '0';
            serial_bus_state         <= eeprom_read;
            drs_eeprom_sector        <= drs_ctl_eeprom_sector(7 downto 0);
          end if;

        -- wait for serdes process
        when wait_serdes =>
          if (serial_count /=0) then
            serial_count <= serial_count;
            if (serdes_state = idle) then
              serial_bus_state <= idle;
              o_drs_dac_cs_n <= '1';
            end if;
          end if;

        -- read from EEPROM to DPRAM
        when eeprom_read =>
          serdes_speed               <= 0;
          case (serial_count) is
            when 0 =>
              o_drs_eeprom_cs_n      <= '0';
              stat_eeprom_spi_command  <= "00000011";  -- diagnostic
              serdes_wdata(31 downto 0)<= "00000011" & drs_eeprom_sector 
                                                     & X"0000"; -- Read page
              serdes_bit_count_m1    <= 31;
              serdes_trig            <= '1';
              dpram_eeprom_reset     <= '1';
            when 2 =>
              if (serdes_state = busy) then
                serial_count <= serial_count;
              end if;
            when 3 =>
              dpram_eeprom_we        <= '0';
              dpram_eeprom_reset     <= '0';
              serdes_wdata           <= (others => '0'); -- Dummy bits
              serdes_bit_count_m1    <= 31; -- read 32 bits
              serdes_trig            <= '1';
            when 5 =>
              if (serdes_state = busy) then
                serial_count         <= serial_count;
              end if;
            when 6 =>
              -- Wolfgang.Rauch@cern.ch -- this has to be rethought
              -- DO NOT
              -- swap bytes since LSB comes first out of the EEPROM and gets
              -- stored then in rdata(31 downto 24)
--              drs_dpram_d_wr(31 downto 24) <= serdes_rdata( 7 downto  0);
--              drs_dpram_d_wr(23 downto 16) <= serdes_rdata(15 downto  8);
--              drs_dpram_d_wr(15 downto  8) <= serdes_rdata(23 downto 16);
--              drs_dpram_d_wr( 7 downto  0) <= serdes_rdata(31 downto 24);
              dpram_eeprom_d_wr <= serdes_rdata;
              dpram_eeprom_addr_next      <= dpram_eeprom_addr + "100";  -- 4 byte addressing
              dpram_eeprom_we   <= '1';

              -- page 0 addr 3 & 7 contains serial number
              if (dpram_eeprom_addr = X"0000" and drs_eeprom_sector = X"00") then
--                 drs_serial_number(7 downto 0)   <= serdes_rdata(31 downto 24);
--                 drs_serial_number(15 downto 8)  <= serdes_rdata(23 downto 16);
                 drs_serial_number   <= serdes_rdata(31 downto 16);
              end if;
             
              -- loop until 32kb read, why only 32k Wolfgang.Rauch@cern.ch ???????????????
              if (dpram_eeprom_addr = X"7FFC") then
                 serial_bus_state    <= idle;
                 drs_eeprom_busy     <= '0';
              end if;
              serial_count           <= 3;
            when others =>
              null;
          end case;

        -- write data from DPRAM to EEPROM
--
-- Wolfgang.Rauch@cern.ch
-- The EEPROM write procedure seems to be broken
-- We'll see later...
--

        when eeprom_write =>
          serdes_speed               <= 0;
          case (serial_count) is

            -- call enable routine
            when 0 =>
              serial_ret_addr        <= 1;
              serial_count           <= 60;

            -- unprotect blocks, will fail if jumper missing
            when 1 =>
              o_drs_eeprom_cs_n      <= '0';
              stat_eeprom_sc_debug      <= "00000001";  -- diagnostic
              stat_eeprom_spi_command   <= "00000001";  -- diagnostic
              serdes_wdata(15 downto 8) <= "00000001";  -- Write status register
              serdes_wdata( 7 downto 0) <= "00000000";  -- no SRWD, no BPx
              serdes_bit_count_m1    <= 15;
              serdes_trig            <= '1';
            when 3 =>
              if (serdes_state = busy) then
                serial_count <= serial_count;
              end if;
            when 4 =>
              o_drs_eeprom_cs_n      <= '1';
              
            -- check status register  
            when 6 =>
              stat_eeprom_sc_debug      <= "00000110";  -- diagnostic
              serial_ret_addr        <= 7;
              serial_count           <= 50;  
            
            -- call enable routine
            when 7 =>
              serial_ret_addr        <= 8;
              serial_count           <= 60;

            -- erase 64 kB block
            when 8 =>
              o_drs_eeprom_cs_n      <= '0';
              stat_eeprom_spi_command   <= "11011000";  -- diagnostic
              serdes_wdata(31 downto 0) <= "11011000" & drs_eeprom_sector 
                                                      & X"0000"; -- Erase block
              serdes_bit_count_m1    <= 31;
              serdes_trig            <= '1';
            when 10 =>
              if (serdes_state = busy) then
                serial_count <= serial_count;
              end if;
            when 11 =>
              o_drs_eeprom_cs_n      <= '1';  
              dpram_eeprom_reset       <= '1';
             
            -- check status register until erase finished
            when 13 =>
              stat_eeprom_sc_debug      <= "00001101";  -- diagnostic
              serial_ret_addr        <= 14;
              serial_count           <= 50;
            
            -- initialize page counter
            when 14 =>
              drs_eeprom_page        <= (others=>'0');
              dpram_eeprom_reset <= '0';

            when 15 =>
              -- call enable routine
              serial_ret_addr        <= 16;
              serial_count           <= 60;

            -- write page (256 bytes)
            when 16 =>
              o_drs_eeprom_cs_n      <= '0';
              -- write page command, use only 1st half of 64 kB sector
              -- why ???? Wolfgang.Rauch@cern.ch
              stat_eeprom_spi_command   <= "10000010";  -- diagnostic
              serdes_wdata(31 downto 0) <= "00000010" & drs_eeprom_sector 
                                         & "0" & drs_eeprom_page(6 downto 0) & X"00";
              serdes_bit_count_m1    <= 31;
              serdes_trig            <= '1';
              drs_eeprom_byte        <= (others=>'0');
            when 18 =>
              if (serdes_state = busy) then
                serial_count <= serial_count;
              end if;
            when 19 =>
              -- write byte
              case (drs_eeprom_byte(1 downto 0)) is
                when "00" =>
                  serdes_wdata(7 downto 0) <= dpram_eeprom_d_rd( 7 downto  0);
                when "01" =>
                  serdes_wdata(7 downto 0) <= dpram_eeprom_d_rd(15 downto  8);
                when "10" =>
                  serdes_wdata(7 downto 0) <= dpram_eeprom_d_rd(23 downto 16);
                when "11" =>
                  serdes_wdata(7 downto 0) <= dpram_eeprom_d_rd(31 downto 24);
                when others =>
                  null;                
              end case;  
              serdes_bit_count_m1    <= 7;
              serdes_trig            <= '1';
              if (drs_eeprom_byte(1 downto 0) = "11") then
                 dpram_eeprom_addr_next      <= dpram_eeprom_addr + "100";  -- 4 byte addressing
                -- dpram_eeprom_inc       <= '1';
              end if;
            when 20 =>
                -- dpram_eeprom_inc          <= '0';  
            when 21 =>
              if (serdes_state = busy) then
                serial_count <= serial_count;
              end if;
            when 22 =>
              drs_eeprom_byte         <= drs_eeprom_byte + 1;
              if (drs_eeprom_byte /= X"FF") then
                serial_count         <= 19; -- loop until 256 bytes written
              end if;
            when 23 =>
              o_drs_eeprom_cs_n      <= '1';
              
            -- check status register
            when 25 =>  
              stat_eeprom_sc_debug   <= "00011001";  -- diagnostic
              serial_ret_addr        <= 26;
              serial_count           <= 50;
              
            -- loop until 128 pages are written  
            -- why only 128 x 256 byte Wolfgang.Rauch@cern.ch ???????????????
            when 26 =>
              drs_eeprom_page        <= drs_eeprom_page + 1;
              if (drs_eeprom_page /= X"7F") then
                serial_count         <= 15;
              end if;

            -- finish write  
            when 27 =>
              drs_eeprom_busy        <= '0';
              serial_bus_state       <= idle;

            -- Wolfgang.Rauch@cern.ch -- the EEPROM stuff has to be rethought
            -- call enable routine
--            when 27 =>
--              serial_ret_addr        <= 28;
--              serial_count           <= 60;
     
            -- remove protection of status register
            when 28 =>
              o_drs_eeprom_cs_n      <= '0';
              stat_eeprom_spi_command   <= "00000001";  -- diagnostic
              serdes_wdata(15 downto 8) <= "00000001";  -- Write status register
              serdes_wdata( 7 downto 0) <= "00011100";  -- remove WPEN
              serdes_bit_count_m1    <= 15;
              serdes_trig            <= '1';
            when 30 =>
              if (serdes_state = busy) then
                serial_count <= serial_count;
              end if;
            when 31 =>
              o_drs_eeprom_cs_n      <= '1';
     
            -- check status register  
            when 33 =>
              serial_ret_addr        <= 34;
              serial_count           <= 50;  
     
            -- call enable routine
            when 34 =>
              serial_ret_addr        <= 35;
              serial_count           <= 60;
     
            -- protect status register and memory
            when 35 =>
              o_drs_eeprom_cs_n      <= '0';
              stat_eeprom_spi_command   <= "00000001";  -- diagnostic
              serdes_wdata(15 downto 8) <= "00000001";  -- Write status register
              serdes_wdata( 7 downto 0) <= "10011100";  -- set WPEN, BP0 - BP2
              serdes_bit_count_m1    <= 15;
              serdes_trig            <= '1';
            when 37 =>
              if (serdes_state = busy) then
                serial_count <= serial_count;
              end if;
            when 38 =>
              o_drs_eeprom_cs_n      <= '1';
     
            -- check status register  
            when 40 =>
              serial_ret_addr        <= 41;
              serial_count           <= 50;  
     
            -- finish write  
            when 41 =>
              drs_eeprom_busy        <= '0';
              serial_bus_state       <= idle;

            -- check status register "subroutine" --------------------------
            when 50 => 
              o_drs_eeprom_cs_n      <= '0';
              stat_eeprom_spi_command   <= "00000101";  -- diagnostic
              serdes_wdata(15 downto 0) <= "00000101" & X"00"; -- Read Status Register
              serdes_bit_count_m1    <= 15;
              serdes_trig            <= '1';
            when 52 =>
              if (serdes_state = busy) then
                serial_count <= serial_count;
              end if;
            when 53 =>
              o_drs_eeprom_cs_n      <= '1';
            when 55 =>
              -- repeat read status until WIP = '0'       
              if (serdes_rdata(0) = '1') then
                serial_count         <= 50; 
              end if;
            when 59 =>
              serial_count           <= serial_ret_addr;
    
            -- enable erase and write "subroutine" -------------------------
            when 60 =>
              o_drs_eeprom_cs_n      <= '0';
              stat_eeprom_spi_command  <= "00000110";  -- diagnostic
              serdes_wdata(7 downto 0) <= "00000110";  -- WREN
              serdes_bit_count_m1    <= 7;
              serdes_trig            <= '1';
            when 62 =>
              if (serdes_state = busy) then
                serial_count <= serial_count;
              end if;
            when 63 =>
              o_drs_eeprom_cs_n      <= '1';
            when 66 =>
              serial_count           <= serial_ret_addr;
    
            when others =>
              null;
          end case;

        when others =>
          serial_bus_state <= idle;

      end case;

    end if;
  end process;

  -- Serializer/Deserializer for serial bus. Upon a serdes_trig,
  -- output serdes_bit_count_m1 bits of serdes_wdata
  proc_serdes: process(I_RESET, drs_serial_clk)
  begin
    if (I_RESET = '1') then

      o_drs_serial_clk      <= '0';
      o_drs_ts_sck          <= '1';
      o_drs_serial_data     <= '0';
      serdes_bit_no         <= 0;
      serdes_state          <= idle;
      serdes_rdata          <= (others => '0');
      stat_serdes_busy      <= '0';
      temp                  <= (others => '0');
      drs_temperature(0)    <= (others => '0');
      drs_temperature(1)    <= X"BEEF";
      drs_temperature(2)    <= X"BABE";
      drs_temperature(3)    <= X"F00D";
      temp_cmd              <= X"C1"; -- temperature register read

    elsif rising_edge(drs_serial_clk) then

      stat_serdes_busy      <= '1'; -- busy unless in idle state 

      case (serdes_state) is

        when idle =>
          stat_serdes_busy      <= '0';
          o_drs_serial_clk      <= '0';
          o_drs_ts_sck          <= '1';
          o_drs_serial_data     <= '0';
          serdes_clk            <= 0;
          serdes_count          <= 0;
          if (serdes_trig = '1') then
            serdes_state          <= busy;
            serdes_bit_no         <= serdes_bit_count_m1;
          end if;
          if (serdes_trig_temp = '1') then
            serdes_state          <= busy_temp;
            serdes_bit_no         <= 7;
          end if;

        when busy =>
          serdes_clk <= serdes_clk + 1;
          if (serdes_clk = serdes_speed) then
            serdes_clk   <= 0;
            serdes_count <= serdes_count + 1;

            case (serdes_count) is
              when 0 =>
                o_drs_serial_clk <= '0';
                o_drs_serial_data <= serdes_wdata(serdes_bit_no);
              when 2 =>
                o_drs_serial_clk <= '1';
              when 3 =>
                serdes_rdata(serdes_bit_no) <= i_drs_eeprom_data;
--                serdes_rdata(serdes_bit_no) <= '0';
                serdes_count <= 0;
                if (serdes_bit_no /= 0) then
                  serdes_bit_no <= serdes_bit_no - 1;
                else
                  serdes_state  <= idle;
                end if;
              when others =>
                null;
              end case;
          end if;

        when busy_temp =>                   -- sequence to read temperature from ADT7301
          serdes_clk <= serdes_clk + 1;
          if (serdes_clk = 9) then -- only every 300ns
            serdes_clk <= 0;

            if (serdes_count < 16) then
              o_drs_ts_sck <= NOT o_drs_ts_sck;
            end if;

            if (o_drs_ts_sck = '0' AND serdes_count < 16) then
              serdes_count <= serdes_count + 1;
              temp         <= temp(temp'high-1 downto 2) & i_drs_serial_data & "00";
            end if;

            if (serdes_count = 16) then
              o_drs_ts_sck  <= '1';
              drs_temperature(0) <= temp(15 downto 3) & "000";
              serdes_state    <= idle;
            end if;
          end if;

        when others => -- should never be entered...
          serdes_state <= idle;
      end case;
    end if;
  end process;

  -- hardware trigger inputs with selectable polarity
  drs_hard_inp(0) <= I_ANA_TRG(0) when drs_ctl_neg_trigger = '0' else not I_ANA_TRG(0);
  drs_hard_inp(1) <= I_ANA_TRG(1) when drs_ctl_neg_trigger = '0' else not I_ANA_TRG(1);
  drs_hard_inp(2) <= I_ANA_TRG(2) when drs_ctl_neg_trigger = '0' else not I_ANA_TRG(2);
  drs_hard_inp(3) <= I_ANA_TRG(3) when drs_ctl_neg_trigger = '0' else not I_ANA_TRG(3);
  drs_hard_inp(4) <= I_ETRG_IN    when drs_ctl_neg_trigger = '0' else not I_ETRG_IN;
  
  -- OR block
  drs_hard_or  <= (drs_hard_inp(0) and drs_ctl_trigger_config(0)) or
                  (drs_hard_inp(1) and drs_ctl_trigger_config(1)) or
                  (drs_hard_inp(2) and drs_ctl_trigger_config(2)) or
                  (drs_hard_inp(3) and drs_ctl_trigger_config(3)) or
                  (drs_hard_inp(4) and drs_ctl_trigger_config(4));
                 
  -- AND block
  drs_hard_and <= '0' when drs_ctl_trigger_config(12 downto 8) = "00000" else
                  ((drs_hard_inp(0) and drs_ctl_trigger_config(8))  or not drs_ctl_trigger_config(8))  and
                  ((drs_hard_inp(1) and drs_ctl_trigger_config(9))  or not drs_ctl_trigger_config(9))  and
                  ((drs_hard_inp(2) and drs_ctl_trigger_config(10)) or not drs_ctl_trigger_config(10)) and
                  ((drs_hard_inp(3) and drs_ctl_trigger_config(11)) or not drs_ctl_trigger_config(11)) and
                  ((drs_hard_inp(4) and drs_ctl_trigger_config(12)) or not drs_ctl_trigger_config(12));
                 
  -- global OR  
  drs_hard_trig <= drs_hard_or or drs_hard_and when drs_ctl_enable_trigger = '1' else GND;
  
  -- flip-flop which toggles with leading edge of hardware trigger
  proc_trg_ff: process(drs_hard_trig, drs_arm_trig)
  begin
    if (drs_arm_trig = '0') then
      drs_trig_ff              <= '0';
    elsif rising_edge(drs_hard_trig) then
      drs_trig_ff              <= '1';
    end if;    
  end process;

  -- output copy of trigger
  O_ETRG_OUT <= drs_trig_ff;

--  -- buffer chain delay for hardware trigger
--  drs_trg_delay(0)             <= drs_trig_ff;
--  delayed_trig_gen: for bit_no in 1 to 2047 generate
--    LUT1_inst : LUT1
--    generic map (
--      INIT => "10")
--    port map (
--      O  => drs_trg_delay(bit_no),
--      I0 => drs_trg_delay(bit_no-1)
--    );
--  end generate;
--
--  -- select delayed hardware trigger
--  drs_trigger                  <= drs_trg_delay(CONV_INTEGER(drs_ctl_delay_sel)*8);

  drs_trigger <= drs_trig_ff;  -- modified by SD, commented out the above delay chain

  -- flip-flop for o_drs_write signal: raise with drs_write_set and lower with trigger
  proc_drs_write_ff: process(drs_trigger, drs_write_set, drs_readout_state)
  begin
    if (drs_readout_state = init or
        drs_readout_state = start_readout) then -- remove o_drs_write also if drs_soft_trig received
      drs_write_ff             <= '0';
    elsif (drs_write_set = '1') then
      drs_write_ff             <= '1';
    elsif rising_edge(drs_trigger) then
      drs_write_ff             <= '0';
    end if;
  end process;
  
  -- hardware trigger: remove o_drs_write asynchronously on rising edge of trigger
  o_drs_write <= drs_write_ff;
                 
  -- Domino chip readout
  proc_drs_readout: process(I_RESET, drs_readout_clk)
  begin
    if (I_RESET = '1') then
      dpram_readout_reset      <= '1';
      dpram_readout_d_wr       <= (others => X"00000000");
      dpram_readout_addr       <= (others => X"000"&"00");
      dpram_readout_addr_next  <= (others => X"000"&"00");
      dpram_readout_we         <= (others => '0');
      o_drs_enable             <= '0';     -- domino waves disabled
      drs_write_set            <= '0';
      o_drs_srin               <= '1';
      o_drs_addr               <= "1111";  -- standby
      drs_readout_state        <= init;
      drs_stat_busy            <= '0';
      drs_stat_running         <= '0';
      drs_stat_idle            <= '0';
      drs_sample_count         <= (others => '0');
      drs_vdd_count            <= (others => '0');
      drs_rd_tmp_count         <= (others => '0');
      drs_reinit_request       <= '1';
      drs_arm_trig             <= '0';
      drs_trigger_syn          <= '0';
      drs_soft_trig            <= '0';
      drs_old_readout_mode     <= '1'; -- "default" mode is "start at stop cell"
      o_drs_adc_clk            <= '0';
--      drs_read_first_chn       <= '0';

    elsif rising_edge(drs_readout_clk) then
      drs_stat_running     <= '0';   -- status register running flag
      drs_stat_idle        <= '0';   -- status register idle flag
      dpram_readout_addr   <=  dpram_readout_addr_next;
      dpram_readout_we     <= (others => '0');
      dpram_pedest_we      <= (others => '0');
      dpram_readout_reset  <= '0';
      dpram_pedest_reset   <= '0';

      -- Memorize a write access to the bit in the control register
      -- that requests a reinitialisation of the DRS readout state
      -- machine (drs_ctl_reinit_trig goes high for only one cycle,
      -- therefore this "trigger" is memorised).
      if (drs_ctl_reinit_trig = '1') then
        drs_reinit_request     <= '1';
      end if;

      -- sample asynchronous trigger
      drs_trigger_syn          <= drs_trigger;
      
      -- remember software trigger
      if (drs_ctl_soft_trig = '1') then
         drs_soft_trig         <= '1';
      end if;

      -- run ADC clock continously
      o_drs_adc_clk            <= not o_drs_adc_clk;
      
      case (drs_readout_state) is

        when init =>
          drs_stat_busy        <= '0';                   
          drs_reinit_request   <= '0';                   
          drs_readout_state    <= idle;
          drs_trigger_bus      <= (others => '0');       
          o_drs_enable         <= '0';                   

        when idle =>
          drs_stat_idle        <= '1';   -- status register idle flag
          drs_stat_busy        <= '0';                   
          drs_arm_trig         <= '0';
          drs_soft_trig        <= '0';
          
          if (drs_ctl_standby_mode = '1') then
            o_drs_addr         <= "1111";  -- standby mode
          else
            o_drs_addr         <= "1011";  -- address read shift register
          end if;  
          o_drs_srin           <= '0';                   
          o_drs_rsrload        <= '0';
          o_drs_srclk          <= '0';
          drs_start_timer      <=  0;
          
--          if (drs_ctl_adc_active = '0') then
--            o_drs_adc_clk      <= '0';
--          end if;
          if (drs_reinit_request = '1') then
            drs_readout_state  <= init;
          elsif (drs_ctl_start_trig = '1') then
            drs_readout_state  <= start_running;
--            drs_stat_busy      <= '1';   -- status reg. busy flag
          end if;

          --  check high byte of drs_ctl_config register          
          if (I_BYTESEL_ARR(10)(3) = '1') then
            drs_readout_state  <= conf_setup;
            o_drs_addr         <= "1100";  -- address config register
          end if;

          -- detect 1 to 0 transition of readout mode
          -- read-out mode = 1 is default, readout starting at stop cell
          -- in read-out mode 0 cells are readout starting at cell 0 
          -- and the procedure at init_rsr has to be invoked
          drs_old_readout_mode <= drs_ctl_readout_mode;
          if (drs_old_readout_mode = '1' and drs_ctl_readout_mode = '0') then
            drs_readout_state  <= init_rsr;
            o_drs_addr         <= "1011";  -- address read shift register
            drs_sr_count       <= 0;
          end if;

        when start_running =>
          o_drs_enable         <= '1';   -- enable and start domino wave
          if (drs_start_timer = 0) then
            drs_write_set      <= '1';   -- set drs_write_ff in proc_drs_write
          else
            drs_write_set      <= '0';
          end if;
        
--          if (drs_ctl_adc_active = '0') then
--            o_drs_adc_clk      <= '0';
--          end if;
          if (drs_reinit_request = '1') then
            drs_readout_state  <= init;
          end if;

          -- do not go to running until at least 1.5 domino revolutions
          drs_start_timer      <= drs_start_timer + 1;
          if (drs_start_timer = 105) then  -- 105 * 30ns = 3.15us
            drs_readout_state  <= running;
            drs_arm_trig       <= '1';  -- arm trigger
          end if;

          
        when running =>
          drs_stat_running      <= '1';   -- status register running flag
        
--          if (drs_ctl_adc_active = '0') then
--            o_drs_adc_clk      <= '0';
--          end if;
          if (drs_reinit_request = '1') then
            drs_readout_state  <= init;
          end if;
          if (drs_ctl_standby_mode = '1') then
            drs_readout_state  <= idle;
          end if;

          -- External trigger or software trigger received or DMODE = 0? If so:
          -- stop domino wave & start readout sequence
          if (drs_trigger_syn = '1' or drs_soft_trig = '1' or drs_ctl_dmode = '0') then
            drs_readout_state  <= start_readout;
            drs_stat_busy      <= '1';   -- status register busy flag
          end if;

        when start_readout =>
          if (drs_ctl_dactive = '0') then 
            o_drs_enable       <= '0';   -- stop domino wave only if dactive=0
          end if;   

          drs_arm_trig         <= '0';   -- disarm trigger
          
          drs_addr             <= drs_ctl_first_chn; -- first channel of drs read-out
          o_drs_addr           <= "1101";  -- address write shift register for readout
          drs_sample_count     <= (others => '0');
          drs_rd_tmp_count     <= (others => '0');
          drs_stop_cell        <= (others => "0000000000");
          drs_stop_wsr         <= (others => "00000000");

--          drs_read_first_chn   <= '1'; -- set flag for first channel action

          -- reset DPRAM (Wolfgang.Rauch@cern.ch: Is this necessary?)
          dpram_readout_reset     <= '1';
          dpram_readout_addr_next  <= (others => X"000"&"00");
          
          drs_readout_state    <= wait_vdd;
          
        when wait_vdd =>
          if (drs_reinit_request = '1') then
            drs_readout_state  <= init;
          end if;
        
          drs_vdd_count     <= drs_vdd_count + 1;
          
          -- wait ~120 us for VDD to stabilize
          if (drs_vdd_count(drs_vdd_count'high) = '1') then
             drs_vdd_count  <= (others => '0');
             drs_readout_state <= adc_sync;
          end if;
          
        when adc_sync =>
          -- synchronize with free running ADC clock
          if (o_drs_adc_clk = '1') then                  -- original value (PSI dev. board)
--          if (o_drs_adc_clk = '0') then                -- DON'T KNOW WHICH ONE IS GOOD (0 OR 1) ????????????????????????????????????????????
             drs_readout_state <= chip_readout;
          end if;
          l1_fifo_wr <= '0';

        when chip_readout =>
          if (drs_reinit_request = '1') then
            drs_readout_state  <= init;
          end if;

          o_drs_srclk          <= '0';
          dpram_readout_reset  <= '0';                   
          
          if (drs_rd_tmp_count(drs_rd_tmp_count'high) = '0') then
            drs_rd_tmp_count   <= drs_rd_tmp_count + 1;
          end if;
          
          if (drs_rd_tmp_count = 0) then
          -- for read-out mode 0 cells are readout starting at cell 0 
          -- the corresponding initialization has already been done
          -- in read-out mode 1 cells are readout starting from the stop position 
          -- this stop position has to be loaded
            if (drs_ctl_readout_mode = '1') then
              o_drs_rsrload    <= '1'; -- load read shift register with stop position
            end if;
          end if;  

          if (drs_rd_tmp_count = 1) then
            o_drs_rsrload      <= '0';
          end if;

-- Wolfgang.Rauch@cern.ch: here we should read out ALL bits of stop_wsr
          if (drs_rd_tmp_count = 2 and drs_addr = drs_ctl_first_chn) then 
            drs_stop_wsr(0)(0)   <= i_drs_srout(0);   -- sample last bit of WSR for first channel DRS 1
            drs_stop_wsr(1)(0)   <= i_drs_srout(1);   -- sample last bit of WSR for first channel DRS 2
            drs_stop_wsr(2)(0)   <= i_drs_srout(2);   -- sample last bit of WSR for first channel DRS 3
            drs_stop_wsr(3)(0)   <= i_drs_srout(3);   -- sample last bit of WSR for first channel DRS 4
--            drs_read_first_chn <= '0'; -- clear flag for first channel action
          end if;
          
          if (drs_rd_tmp_count = 2) then
            o_drs_addr         <= drs_addr;      -- select channel for readout
          end if;

--          if (drs_rd_tmp_count > 13 and drs_sample_count < 1016) then
          if (drs_rd_tmp_count > 13 and drs_sample_count < 1006) then  -- modified by SD, from <1016 to <1006 due to a different ADC latency (16 instead of 7)
            o_drs_srclk        <= not o_drs_srclk; -- toggle drs serial clock
          end if;

          if (drs_rd_tmp_count > 13 and drs_rd_tmp_count < 36) then  -- Wolfgang.Rauch@cern.ch: why 35-14+1=22 (11 with drs_clk = 1) ?
            if (o_drs_srclk = '1') then
              drs_stop_cell(0)(0) <= i_drs_srout(0);
              drs_stop_cell(0)(9 downto 1) <= drs_stop_cell(0)(8 downto 0);
              drs_stop_cell(1)(0) <= i_drs_srout(1);
              drs_stop_cell(1)(9 downto 1) <= drs_stop_cell(1)(8 downto 0);
              drs_stop_cell(2)(0) <= i_drs_srout(2);
              drs_stop_cell(2)(9 downto 1) <= drs_stop_cell(2)(8 downto 0);
              drs_stop_cell(3)(0) <= i_drs_srout(3);
              drs_stop_cell(3)(9 downto 1) <= drs_stop_cell(3)(8 downto 0);
            end if;
          end if;

          -- ADC delivers data at its outputs with 7 clock cycles delay
          -- with respect to its external clock pin
--          if (drs_rd_tmp_count > 29 and o_drs_adc_clk = '1') then
          if (drs_rd_tmp_count > 49 and o_drs_adc_clk = '1') then  -- modified by SD, from >29 to >49 due to a different ADC latency (16 instead of 7)
            drs_sample_count   <= drs_sample_count + 1;
            
-- start Wolfgang.Rauch@cern.ch
            loop_readout: for i in 0 to 3 loop
            if (drs_sample_count(0) = '0') then -- lsb=0, even
              -- even cells: WORD0, WORD2, WORD4, ...
              dpram_readout_d_wr(i)(15 downto 12) <= drs_addr; -- diagnostic
              dpram_readout_d_wr(i)(11 downto 0)  <= i_drs_adc(i)(11 downto 0);  -- sample ADC data
            else -- lsb=1, odd
              -- odd cells: WORD1, WORD3, WORD5, ...
              dpram_readout_d_wr(i)(31 downto 28) <= drs_addr; -- diagnostic
              dpram_readout_d_wr(i)(27 downto 16) <= i_drs_adc(i)(11 downto 0);  -- sample ADC data

              dpram_readout_addr_next(i)          <= dpram_readout_addr(i) + "100";  -- 4 byte addressing
              dpram_readout_we(i)                 <= '1';  -- write 32 bits to DPRAM
            end if;
            end loop loop_readout;
-- end Wolfgang.Rauch@cern.ch
            
          end if;

          if (drs_sample_count = 1024) then
            drs_sample_count   <= (others => '0');
            drs_rd_tmp_count   <= (others => '0');
            o_drs_srclk        <= '0';

            -- write stop cell into register
            drs_stat_stop_cell <= drs_stop_cell;
            drs_stat_stop_wsr  <= drs_stop_wsr;

            -- increment channel address
            drs_addr           <= drs_addr + 1;

            -- re-sync ADC
            drs_readout_state  <= adc_sync;

            -- All channels of DRS chips read ?
            if (drs_addr = drs_ctl_last_chn) then
--               drs_readout_state <= trailer;
-- header or trailer should be produced by reading csr registers from the host
               drs_readout_state <= done;
            end if;

          end if;

        when done =>
          drs_readout_state    <= idle;
--          drs_stat_busy        <= '0';
          dpram_readout_we     <= (others => '0');
          drs_write_set        <= '1';   -- set drs_write_ff in proc_drs_write
                                         -- to keep chip "warm"

        -- set-up of configuration register        
        when conf_setup =>
-- address of config reg "1100" already loaded
          o_drs_srclk          <= '1';
          drs_sr_count         <= 0;
          drs_sr_reg           <= drs_ctl_config;
          drs_readout_state    <= conf_strobe;
          o_drs_srin           <= drs_ctl_config(7);
        
        when conf_strobe =>  
          drs_sr_count         <= drs_sr_count + 1;
          o_drs_srclk          <= not o_drs_srclk; -- toggle drs serial clock

          if (o_drs_srclk = '1') then
             drs_sr_reg(7 downto 1)<= drs_sr_reg(6 downto 0);
          else
             o_drs_srin        <= drs_sr_reg(7);
          end if;
          
          if (drs_sr_count = 14) then
             drs_readout_state <= wsr_addr;
          end if;
          
        -- change address without changing clock  
        when wsr_addr =>
          o_drs_addr           <= "1101";  -- address write shift register
          drs_readout_state    <= wsr_setup;

        -- set-up of write shift register
        when wsr_setup =>
          o_drs_srclk          <= '1';
          drs_sr_count         <= 0;
          drs_sr_reg           <= drs_ctl_chn_config;
          drs_readout_state    <= wsr_strobe;
          o_drs_srin           <= drs_ctl_chn_config(7);
        
        when wsr_strobe =>  
          drs_sr_count         <= drs_sr_count + 1;
          o_drs_srclk          <= not o_drs_srclk; -- toggle drs serial clock

          if (o_drs_srclk = '1') then
             drs_sr_reg(7 downto 1) <= drs_sr_reg(6 downto 0);
          else
             o_drs_srin        <= drs_sr_reg(7);
          end if;
          
          if (drs_sr_count = 14) then
             drs_readout_state <= idle;
          end if;

        -- initialize read shift register
        when init_rsr =>
          if (o_drs_srclk = '1') then
            drs_sr_count       <= drs_sr_count + 1;
          end if;  
          o_drs_srclk          <= not o_drs_srclk; -- toggle serial clock
        
          if (drs_sr_count = 1023 and o_drs_srclk = '0') then
            o_drs_srin         <= '1';
          end if;
          
          if (drs_sr_count = 1024 and o_drs_srclk = '0') then
            o_drs_srin         <= '0';
            drs_readout_state  <= idle;
            o_drs_srclk        <= '0';             -- toggle serial clock
          end if;
          
        when others =>
          drs_readout_state    <= init;

      end case;
    end if;
  end process;


  -- Reference clock used for frequency counter
  
  proc_1hzclk: process(I_RESET, I_DRS_READOUT_CLK)
  begin
    if (I_RESET = '1') then
      drs_1hz_counter(31 downto 0)          <= (others => '0');
		drs_1hz_clock                         <= '0';
		scaler_reset                          <= (others => '1');
		scaler_ff_reset                       <= (others => '1');
    elsif rising_edge(I_DRS_READOUT_CLK) then
      drs_1hz_counter                       <= drs_1hz_counter - 1; -- count down
		scaler_reset                          <= (others => '0');
		scaler_ff_reset                       <= (others => '0');
      
      -- toggle refclk if timer expires      
      if (drs_1hz_counter(drs_1hz_counter'high) = '1') then
        drs_1hz_clock                       <= not drs_1hz_clock;
-- Wolfgang.Rauch@cern.ch: careful, clock may be 66 MHz
--        drs_1hz_counter(31 downto 0)        <= X"0016E35F";     -- 1499999, I_CLK33 is actually a 30 MHz clock
        drs_1hz_counter(31 downto 0)        <= X"00325A9F";       -- 3299999, for 66 MHz
		  
		  scaler_ff_reset                     <= (others => '1'); -- reset scaler_ff once every 100ms cycle
        loop_scaler_reset : for i in 0 to 5 loop
          if (scaler_ff(i) = '0') then                          -- no activity since last cycle?     
		       scaler_reset(i)                <= '1';             -- force clear scaler register
		    end if;
        end loop;

		  if (scaler_ff(0) = '0') then                            -- no activity since last cycle?     
		    scaler_reset(0)                   <= '1';             -- force clear scaler register
		  end if;
		  
      end if;  
    end if;
  end process;
 
  
  -- scalers for channels 1-4 & trigger & ext. clock 
  gen_scaler_ff : for i in 0 to 5 generate
  begin

  -- RS flip-flop to register any scaler activity, needed to clear the scaler for 0 Hz
  proc_scaler_ff: process(scaler_ff_reset(i), scaler_clock(i))
  begin
    if (scaler_ff_reset(i) = '1') then
      scaler_ff(i) <= '0';
    elsif rising_edge(scaler_clock(i)) then
      scaler_ff(i) <= '1';
    end if;		
  end process;
    
  -- scaler process	 
  proc_drs_scaler: process(scaler_reset(i), scaler_clock(i))
  begin
    if (scaler_reset(i) = '1') then
      scaler(i)                             <= (others => '0');
      O_STATUS_REG_ARR(10+i)                <= (others => '0');
	 	drs_1hz_latch1(i)                     <= '0';
	 	drs_1hz_latch2(i)                     <= '0';
	 	drs_1hz_latch3(i)                     <= '0';
    elsif rising_edge(scaler_clock(i)) then
      scaler(i)                             <= scaler(i) + 1;
		drs_1hz_latch1(i)                     <= drs_1hz_clock;      -- sample asynchronous clock
		drs_1hz_latch2(i)                     <= drs_1hz_latch1(i);
		drs_1hz_latch3(i)                     <= drs_1hz_latch2(i);
      
      -- toggle refclk if timer expires      
      if (drs_1hz_latch1(i) = '1' and drs_1hz_latch2(i) = '0' and drs_1hz_latch3(i) = '0') then
		  scaler(i)                           <= scaler(i);
        O_STATUS_REG_ARR(10+i)              <= scaler(i);
      end if;  
      if (drs_1hz_latch1(i) = '1' and drs_1hz_latch2(i) = '1' and drs_1hz_latch3(i) = '0') then
        scaler(i)                           <= X"00000001"; -- we loose one clock cycle, so add it back here
      end if;  
    end if;
  end process;

  end generate gen_scaler_ff;
  
  scaler_clock(0) <= I_ANA_TRG(0);
  scaler_clock(1) <= I_ANA_TRG(1);
  scaler_clock(2) <= I_ANA_TRG(2);
  scaler_clock(3) <= I_ANA_TRG(3);
  scaler_clock(4) <= drs_hard_trig;
  scaler_clock(5) <= '0';

  -- debugging

  -- timing calibration
  O_TCA_CTRL <= drs_ctl_tca_ctrl;
  O_TCA_SEL  <= drs_ctl_tcal_source;
--  O_CAL      <= not drs_ctl_acalib;
  O_CAL      <= drs_ctl_acalib;

end arch;
