----------------------------------------------------------------------------------
-- Company.......: Universit de Genve - DPNC
-- Author........: SD
-- File..........: ledpulse.vhd
-- Description...: Generates a 20ms pulse for LED
-- Project Name..: DPNC342 - 32-channel DRS4 Acquisition Board - NA61 New trigger
-- Created.......: October 17, 2014
-- Version.......: 0.1
--
-- Revision History:
-- October 17, 2014: file creation
----------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;

entity ledpulse is
  port(PULSE  : in  std_logic;
       LEDOUT : out std_logic;
       CLK    : in  std_logic;
       RESET  : in  std_logic
      );
end ledpulse;

--------------------------------------------------------------------------------
architecture bhv of ledpulse is

type state_type is (IDLE,LEDON,WAIT_LATENCY);
signal state : state_type;

subtype cnt_type is integer range 0 to 2097151;
-- constant LED_LATENCY : cnt_type := 1500000;   -- 20 ms @ CLK=75 MHz
-- constant LED_LATENCY : cnt_type := 600000;    -- 20 ms @ CLK=30 MHz
-- constant LED_LATENCY : cnt_type := 800000;    -- 20 ms @ CLK=40 MHz
constant LED_LATENCY : cnt_type := 2000000;   -- 20 ms @ CLK=100 MHz

begin

ledctl:process(CLK,RESET)
variable cnt : cnt_type;
begin
  if (RESET='1') then
    LEDOUT <= '0';
    cnt := 0;
    state <= IDLE;

  elsif (CLK'event AND CLK='1') then

    case state is

      when IDLE =>
        LEDOUT <= '0';
        if (PULSE='1') then
          state <= LEDON;
        else
          state <= IDLE;
        end if;

      when LEDON =>
        LEDOUT <= '1';
        cnt := 0;
        if (PULSE='1') then
          state <= LEDON;
        else
          state <= WAIT_LATENCY;
        end if;

      when WAIT_LATENCY =>
        if (cnt=LED_LATENCY) then
          if (PULSE='1') then
            state <= LEDON;
          else
            state <= IDLE;
          end if;
        else
          cnt := cnt + 1;
          state <= WAIT_LATENCY;
        end if;

    end case;
  end if;
end process ledctl;

--------------------------------------------------------------------------------

end bhv;
